// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/tx_proc.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

/*
*
*	This module performs the work associated with processing acquired data.
*	This could mean anything from writing the data to a file or device to
*	analysis or evaluation on the fly. This module is expected to start one
*	or more threads to perform this work rather than performing the work
*	inline within the call to get things started.
*
*/
#include <stddef.h>
#include <stdio.h>

#include "tx_proc.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* variables	*************************************************************/

static FILE*	_file;



/*****************************************************************************
*
*	Function:	_dispense_data
*
*	Purpose:
*
*		Implement the operations that do something with the data we're handed.
*		Any and all errors are reported via the "data" structure.
*
*	Arguments:
*
*		data	The process data we to use.
*
*		bm		The data transfer buffer we're to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _dispense_data(proc_data_t* data, gsc_util_buf_man_t* bm)
{
	char*	ptr;
	U32		tx;
	size_t	st;
	U32		status;

	switch (data->operation)
	{
		default:
		case TX_OP_DISCARD:

			bm->offset	= 0;
			bm->count	= 0;	// We consumed everything.
			data->bytes	+= bm->size;
			break;

		case TX_OP_SAVE:

			ptr	= ((char*) bm->buffer) + bm->offset;
			st	= fwrite(ptr, bm->count, 1, _file);

			if (st != 1)
			{
				data->err	= 1;
				sprintf(data->text,
						"fwrite(0x%lX, 0x%lX, 0x%lX, 0x%lX) did not complete as expected",
						(long) ptr,
						(long) bm->count,
						(long) 1,
						(long) _file);
				gsc_buf_man_stop(TRUE, NULL);
			}
			else
			{
				data->bytes	+= bm->count;
				bm->offset	= 0;
				bm->count	= 0;	// We consumed everything.
			}

			break;

		case TX_OP_XMIT:

			ptr	= ((char*) bm->buffer) + bm->offset;
			status	= hpdi32_write(data->dev, ptr, bm->count, &tx);

			if ((status != GSC_SUCCESS) || (tx != bm->count))
			{
				data->err	= 1;
				sprintf(data->text,
						"hpdi32_write(0x%lX, 0x%lX, %ld, %ld): status = 0x%lX",
						(long) data->dev,
						(long) ptr,
						(long) bm->count,
						(long) tx,
						(long) status);
				gsc_buf_man_stop(TRUE, NULL);
			}
			else
			{
				data->bytes	+= bm->count;
				bm->offset	= 0;
				bm->count	= 0;	// We consumed everything.
			}

			break;
	}
}



/*****************************************************************************
*
*	Function:	_tx_process
*
*	Purpose:
*
*		Perform processing on data received.
*
*	Arguments:
*
*		arg		A pointer to the process data we to use.
*
*	Returned:
*
*		0		We don't use this.
*
*****************************************************************************/

static  U32 _tx_process(void* arg)
{
	gsc_util_buf_man_t	bm;
	proc_data_t*		data	= (proc_data_t*) arg;
	int					end;
	int					errs;

	for (;;)
	{
		data->bytes		= 0;
		data->ms_begin	= gsc_os_time_delta_ms();

		if (data->operation == TX_OP_SAVE)
		{
			_file	= fopen(data->file_name, "wb");

			if (_file == NULL)
			{
				data->err	= TRUE;
				strcpy(data->text, "fopen() failed");
				break;
			}
		}
		else
		{
			_file	= NULL;
		}

		errs	= process_thread_sync(data);

		if (errs)
		{
			data->err	= TRUE;
			sprintf(data->text, "process_thread_sync() returned %d", errs);
			break;
		}

		for (;;)
		{
			if (data->stop)
			{
				strcpy(data->text, "stop requested");
				break;
			}

			errs	= gsc_buf_man_request_data(&bm);

			if (errs)
			{
				data->err	= TRUE;
				strcpy(data->text, "gsc_buf_man_request_data() failed");
				break;
			}

			if ((bm.buffer == NULL) && (bm.eof == 0))
			{
				strcpy(data->text, "stop requested");
				break;
			}

			if (bm.buffer)
			{
				_dispense_data(data, &bm);

				if (data->err)
				{
					gsc_buf_man_release_buffer(&bm);
					break;
				}
			}

			end		= bm.eof;
			errs	= gsc_buf_man_release_buffer(&bm);

			if (errs)
			{
				data->err	= TRUE;
				strcpy(data->text, "gsc_buf_man_release_buffer() failed");
				break;
			}

			if (end)
			{
				strcpy(data->text, "end of file");
				break;
			}

		}

		break;
	}

	if (_file)
		fclose(_file);

	data->ms_end	= gsc_os_time_delta_ms();
	data->done		= TRUE;
	return(0);
}



/*****************************************************************************
*
*	Function:	tx_process_start
*
*	Purpose:
*
*		Start the process/thread that does something with acquired data.
*
*	Arguments:
*
*		data	Data for beginning and ending our processing.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int tx_process_start(proc_data_t* data)
{
	int				errs;
	gsc_os_thread_t	thread;

	gsc_label_text(data->name);

	if (data == NULL)
	{
		errs	= gsc_label_fail_text(1, "tx_process_start(): NULL argument.");
	}
	else if (data->dev == NULL)
	{
		errs	= gsc_label_fail_text(1, "tx_process_start(): device handle is NULL");
	}
	else
	{
		switch (data->operation)
		{
			default:

				data->operation	= TX_OP_DISCARD;

			case TX_OP_DISCARD:

				strcpy(data->op_title, "Discarding Data.");
				break;

			case TX_OP_SAVE:

				sprintf(data->op_title, "Saving data to disk: %s", data->file_name);
				break;

			case TX_OP_XMIT:

				strcpy(data->op_title, "Transmitting data to device.");
				break;

		}

		printf("%s\n", data->op_title);
		gsc_label_level_inc();
		gsc_label_text("Starting");
		errs	= gsc_os_thread_create(&thread, _tx_process, data);

		if (errs)
			gsc_label_fail_text(1, "gsc_os_thread_create() failed");
		else
			printf("PASS\n");

		gsc_label_level_dec();
	}

	return(errs);
}


