// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/align.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

/*****************************************************************************
*
*	Implement various control alignment services.
*
*****************************************************************************/

#include <windows.h>

#include "align.h"



/*****************************************************************************
*
*	Function:	align_control
*
*	Purpose:
*
*		Perform alignment of one control with a reference.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		id		The id of the control to move.
*
*		ref		The id of the reference control.
*
*		flags	The flags that govern the alignment.
*
*		value	Any change to the basic alignment.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void align_control(HWND dlg, int id, int ref, UINT flags, int value)
{
	HWND	control;
	RECT	dest;
	int		h;
	RECT	src;
	int		w;

	if (ref)
	{
		control	= GetDlgItem(dlg, ref);
		GetWindowRect(control, &src);
		ScreenToClient(dlg, (POINT*) &src.left);
		ScreenToClient(dlg, (POINT*) &src.right);
	}
	else
	{
		GetClientRect(dlg, &src);
	}

	control	= GetDlgItem(dlg, id);
	GetWindowRect(control, &dest);
	ScreenToClient(dlg, (POINT*) &dest.left);
	ScreenToClient(dlg, (POINT*) &dest.right);
	h	= RECT_HEIGHT(dest);
	w	= RECT_WIDTH(dest);

	if (flags & ALIGN_COPY)
		dest	= src;

	if (flags == ALIGN_W)
		dest.right	= dest.left + value;

	if (flags == ALIGN_H)
		dest.bottom	= dest.top + value;

	if (flags & (ALIGN_L | ALIGN_L_L | ALIGN_L_C | ALIGN_L_R))
	{
		if (flags & ALIGN_L)
			dest.left	= value;
		else if (flags & ALIGN_L_L)
			dest.left	= src.left + value;
		else if (flags & ALIGN_L_C)
			dest.left	= ((src.left + src.right) / 2) + value;
		else if (flags & ALIGN_L_R)
			dest.left	= src.right + value;

		if (flags & ALIGN_W)
			dest.right	= dest.left + w;
	}

	if (flags & (ALIGN_HC_L | ALIGN_HC_C | ALIGN_HC_R))
	{
		if (flags & ALIGN_HC_L)
			dest.left	= src.left - (w / 2) + value;
		else if (flags & ALIGN_HC_C)
			dest.left	= ((src.left + src.right) / 2) - (w / 2) + value;
		else if (flags & ALIGN_HC_R)
			dest.left	= src.right - (w / 2) + value;

		dest.right	= dest.left + w;
	}

	if (flags & (ALIGN_R | ALIGN_R_L | ALIGN_R_C | ALIGN_R_R))
	{
		if (flags & ALIGN_R)
			dest.right	= value;
		else if (flags & ALIGN_R_L)
			dest.right	= src.left + value;
		else if (flags & ALIGN_R_C)
			dest.right	= ((src.left + src.right) / 2) + value;
		else if (flags & ALIGN_R_R)
			dest.right	= src.right + value;

		if (flags & ALIGN_W)
			dest.left	= dest.right - w;
	}

	if (flags & (ALIGN_T | ALIGN_T_T | ALIGN_T_C | ALIGN_T_B))
	{
		if (flags & ALIGN_T)
			dest.top	= value;
		else if (flags & ALIGN_T_T)
			dest.top	= src.top + value;
		else if (flags & ALIGN_T_C)
			dest.top	= ((src.top + src.bottom) / 2) + value;
		else if (flags & ALIGN_T_B)
			dest.top	= src.bottom + value;

		if (flags & ALIGN_H)
			dest.bottom	= dest.top + h;
	}

	if (flags & (ALIGN_VC_T | ALIGN_VC_C | ALIGN_VC_B))
	{
		if (flags & ALIGN_VC_T)
			dest.top	= src.top - (h / 2) + value;
		else if (flags & ALIGN_VC_C)
			dest.top	= ((src.top + src.bottom) / 2) - (h / 2) + value;
		else if (flags & ALIGN_VC_B)
			dest.top	= src.bottom - (h / 2) + value;

		dest.bottom	= dest.top + h;
	}

	if (flags & (ALIGN_B | ALIGN_B_T | ALIGN_B_C | ALIGN_B_B))
	{
		if (flags & ALIGN_B)
			dest.bottom	= value;
		else if (flags & ALIGN_B_T)
			dest.bottom	= src.top + value;
		else if (flags & ALIGN_B_C)
			dest.bottom	= ((src.top + src.bottom) / 2) + value;
		else if (flags & ALIGN_B_B)
			dest.bottom	= src.bottom + value;

		if (flags & ALIGN_H)
			dest.top	= dest.bottom - h;
	}

	MoveWindow(	control,
				dest.left,
				dest.top,
				RECT_WIDTH(dest),
				RECT_HEIGHT(dest),
				FALSE);
}



/*****************************************************************************
*
*	Function:	align_control_list
*
*	Purpose:
*
*		Perform alignment of a list of controls.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		list	The data set to process.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void align_control_list(HWND dlg, const align_t* list)
{
	for (; list->id; list++)
		align_control(dlg, list->id, list->ref, list->flags, list->value);
}



/*****************************************************************************
*
*	Function:	align_control_width
*
*	Purpose:
*
*		Change the control's width per its content and return the width to the
*		caller.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		id		The id of the control to move.
*
*		font	The font to use for the work.
*
*	Returned:
*
*		The width of the control's text, in pixels.
*
*****************************************************************************/

int align_control_width(HWND dlg, int id, HFONT font)
{
	char	buf[1024];
	HWND	control;
	HDC		dc;
	int		len;
	SIZE	size;
	RECT	rect;

	control	= GetDlgItem(dlg, id);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);

	dc		= GetDC(control);
	len		= GetWindowText(control, buf, sizeof(buf));
	font	= SelectObject(dc, font);
	GetTextExtentPoint32(dc, buf, len, &size);
	SelectObject(dc, font);
	ReleaseDC(control, dc);

	rect.right	= rect.left + size.cx;

	MoveWindow(	control,
				rect.left,
				rect.top,
				RECT_WIDTH(rect),
				RECT_HEIGHT(rect),
				FALSE);
	return(size.cx);
}


