// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/config_save.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the Configuration Save option.
*
*****************************************************************************/

#include <windows.h>
#include <process.h>
#include <stdio.h>

#include "config.h"
#include "align.h"
#include "copy.h"
#include "gui.h"
#include "ini.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	CFG_SECTION_NAME	"configuration"
#undef	INI_SECTION_NAME
#define	INI_SECTION_NAME	CFG_SECTION_NAME ":save"



/* typedefs	*****************************************************************/

typedef struct
{
	int						abort;
	int						active;
	device_t*				device;
	char					dir[_MAX_PATH];
	HWND					dlg;
	const config_item_t*	list;
	TCHAR					name[_MAX_PATH];
	size_t					size;
	U32						status;
	char					status_local[1024];

	struct
	{
		HWND	dlg;
		DWORD	id;		// The id of the original thread.
	} thread;

} config_save_t;



/* prototypes	*************************************************************/

static	LRESULT	_wm_size(HWND dlg);



/*****************************************************************************
*
*	Function:	_show_window_status
*
*	Purpose:
*
*		Update the window to show the current status;
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		msg		The message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _show_window_status(HWND dlg, config_save_t* data)
{
	if (data->status != GSC_SUCCESS)
		error_status(dlg, IDC_CFG_FILE_STATUS, data->status);
	else if (data->status_local[0])
		SetDlgItemText(dlg, IDC_CFG_FILE_STATUS, data->status_local);
	else
		SetDlgItemText(dlg, IDC_CFG_FILE_STATUS, NULL);
}



/*****************************************************************************
*
*	Function:	_add_line
*
*	Purpose:
*
*		Add a line of text to the list box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		text		The text of the line to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _add_line(HWND dlg, const char* text)
{
	HWND	control		= GetDlgItem(dlg, IDC_CFG_FILE_LIST);
	int		count;

	SendMessage(control, LB_ADDSTRING, 0, (LPARAM) text);
	count	= (int) SendMessage(control, LB_GETCOUNT, 0, 0);
	SendMessage(control, LB_SETCURSEL, (WPARAM) (count - 1), 0);
}



/*****************************************************************************
*
*	Function:	_parm_value_to_text
*
*	Purpose:
*
*		Convert a parameter value to text.
*
*	Arguments:
*
*		entry		The structure to process.
*
*		value		The value to convert.
*
*		dest		The text representation is put here. It is assumed
*					that this is plenty big enough.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _parm_value_to_text(
	const parm_entry_t*	entry,
	U32					value,
	char*				dest)
{
	size_t						i;
	const value_list_entry_t*	list;

	sprintf(dest, "0x%lX", (long) value);

	if (entry->values->type == VALUE_TYPE_LIST)
	{
		list	= entry->values->u.list->list;
		i		= entry->values->u.list->size;

		for (; i--;)
		{
			if (value == list[i].value)
			{
				strcpy(dest, list[i].value_text);
				break;
			}
		}
	}
}



/*****************************************************************************
*
*	Function:	_process_parm_entry_t
*
*	Purpose:
*
*		Process a single parm_entry_t structure.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*		list		The list containing the below entry to process.
*
*		entry		The structure to process.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _process_parm_entry_t(
	config_save_t*		data,
	const parm_list_t*	list,
	const parm_entry_t*	entry)
{
	char		buf[256];
	HWND		control	= GetDlgItem(data->dlg, IDC_CFG_FILE_STATUS);
	char		key[128];
	U32			status;
	U32			u32;
	char		value[128];

	strcpy(key, entry->parm_text);
	strcat(key, "_");

	if (list->suffix)
		strcat(key, list->suffix);

	if (entry->suffix)
		strcat(key, entry->suffix);

	strcat(key, "_SET");
	sprintf(buf, "Processing %s ...", key);
	SetWindowText(control, buf);

	status	= hpdi32_config(data->device->handle,
							entry->parm,
							list->which | entry->which,
							GSC_NO_CHANGE,
							&u32);
	_parm_value_to_text(entry, u32, value);
	sprintf(buf, "%s=%s", key, value);
	_add_line(data->dlg, buf);

	if (status == GSC_SUCCESS)
		ini_str_set(data->name, CFG_SECTION_NAME, key, value);
	else
		_add_line(data->dlg, "ACCESS ERROR FOR THE ABOVE PARAMETER.");

	return(status);
}



/*****************************************************************************
*
*	Function:	_process_config_item_t
*
*	Purpose:
*
*		Process a single config_item_t structure.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*		item		The structure to process.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _process_config_item_t(
	config_save_t*			data,
	const config_item_t*	item)
{
	size_t						i;
	const parm_list_t*			list;
	const config_parm_list_t*	parm;
	U32							status	= GSC_SUCCESS;

	if (item->type == TYPE_PARM_LIST)
	{
		parm	= (const config_parm_list_t*) item->vp;
		list	= parm->list;

		for (i = 0; i < parm->list->size; i++)
		{
			status	= _process_parm_entry_t(data, list, list->list[i]);

			if (status != GSC_SUCCESS)
				break;

			if (data->abort)
				break;
		}
	}

	if (status != GSC_SUCCESS)
		data->status	= status;
}



/*****************************************************************************
*
*	Function:	_save_thread
*
*	Purpose:
*
*		Implement the Configuration Save service.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		0		We always return 0.
*
*****************************************************************************/

static unsigned WINAPI _save_thread(LPVOID pvArg)
{
	char			buf[1024];
	HWND			control;
	LRESULT			count;
	config_save_t*	data	= (config_save_t*) pvArg;
	HDC				dc;
	size_t			i;
	int				j;
	TEXTMETRIC		tm;
	DWORD			thread_id;
	int				width;

	data->abort		= FALSE;
	data->active	= TRUE;
	thread_id		= GetCurrentThreadId();
	AttachThreadInput(thread_id, data->thread.id, TRUE);

	ctrl_menu_set(data->thread.dlg, IDC_CFG_FILE_ABORT, CTRL_ENABLE | CTRL_FOCUS);
	ctrl_menu_set(data->thread.dlg, IDC_CFG_FILE_AGAIN, CTRL_DISABLE);
	ctrl_menu_set(data->thread.dlg, IDCANCEL, CTRL_DISABLE);

	control	= GetDlgItem(data->dlg, IDC_CFG_FILE_LIST);
	SendMessage(control, LB_RESETCONTENT, 0, 0);

	for (i = 0; i < data->size; i++)
	{
		_process_config_item_t(data, &data->list[i]);

		if (data->status != GSC_SUCCESS)
			break;

		if (data->abort)
			break;
	}

	ctrl_menu_set(data->thread.dlg, IDCANCEL, CTRL_ENABLE | CTRL_FOCUS);
	ctrl_menu_set(data->thread.dlg, IDC_CFG_FILE_AGAIN, CTRL_ENABLE);
	ctrl_menu_set(data->thread.dlg, IDC_CFG_FILE_ABORT, CTRL_DISABLE);

	count	= SendMessage(control, LB_GETCOUNT, 0, 0);
	width	= 0;
	dc		= GetDC(control);
	GetTextMetrics(dc, &tm);
	ReleaseDC(control, dc);

	for (i = 0; i < (size_t) count; i++)
	{
		SendMessage(control, LB_GETTEXT, (WPARAM) i, (LPARAM) buf);
		j	= (int) strlen(buf);

		if (width < j)
			width	= j;
	}

	width	= (width + 2) * tm.tmAveCharWidth;
	SendMessage(control, LB_SETHORIZONTALEXTENT, (WPARAM) width, 0);

	if (data->abort)
	{
		strcpy(data->status_local, "Operation aborted.");
	}
	else if (data->status == GSC_SUCCESS)
	{
		SendMessage(control, LB_SETCURSEL, 0, 0);
		SendMessage(control, LB_SETCURSEL, (WPARAM) -1, 0);
		SendMessage(control, LB_SETTOPINDEX, 0, 0);
		strcpy(data->status_local, "Done.");
	}

	_show_window_status(data->thread.dlg, data);
	data->active	= FALSE;
	return(0);
}



/*****************************************************************************
*
*	Function:	_idc_cfg_file_again
*
*	Purpose:
*
*		Respond to activity on the AGAIN option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cfg_file_again(HWND dlg)
{
	config_save_t*	data;
	unsigned		id;
	long			thread;

	data	= (config_save_t*) GET_WINDOW_USER_DATA(dlg);
	data->status			= GSC_SUCCESS;
	data->status_local[0]	= 0;
	data->thread.dlg		= dlg;
	data->thread.id			= GetCurrentThreadId();
	thread	= (long) _beginthreadex(NULL, 0, _save_thread, data, 0, &id);

	if (thread == -1L)
	{
		strcpy(data->status_local, "Unable to initiate operation.");
		_show_window_status(dlg, data);
	}
}



/*****************************************************************************
*
*	Function:	_idc_cfg_file_list
*
*	Purpose:
*
*		Respond to activity in the list box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cfg_file_list(HWND dlg, WPARAM wParam, config_save_t* data)
{
	WORD	code	= HIWORD(wParam);

	if ((code == LBN_SELCHANGE) && (!data->active))
	{
		data->status			= GSC_SUCCESS;
		data->status_local[0]	= 0;
		config_show_entry_c_code(	dlg,
									data->device,
									data->list,
									data->size,
									data->status_local);
		_show_window_status(dlg, data);
	}
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id					flags						*/
		{ -1,					COPY_EOL | COPY_UNDERLINE	},

		{ IDC_CFG_FILE_TITLE,	0							},
		{ IDC_CFG_FILE_NAME,	COPY_EOL | COPY_INDENT		},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_CFG_FILE_LIST,	COPY_EOL | COPY_LIST_BOX	},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_CFG_FILE_AGAIN,	COPY_EOL | COPY_DISABLED	},
		{ IDC_CFG_FILE_ABORT,	COPY_EOL | COPY_DISABLED	},
		{ IDCANCEL,				COPY_EOL | COPY_DISABLED	},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_CFG_FILE_STATUS,	COPY_EOL					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Process a request to close the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	win_pos_save(dlg, INI_SECTION_NAME);
	EndDialog(dlg, 0);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	config_save_t*	data;
	int				id		= LOWORD(wParam);
	LRESULT			ret		= TRUE;

	data	= (config_save_t*) GET_WINDOW_USER_DATA(dlg);

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_CFG_FILE_ABORT:

			data->abort	= TRUE;
			break;

		case IDC_CFG_FILE_AGAIN:

			_idc_cfg_file_again(dlg);
			break;

		case IDC_CFG_FILE_LIST:

			_idc_cfg_file_list(dlg, wParam, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages's long size parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND dlg, LPARAM lParam)
{
	HWND		control;
	MINMAXINFO*	mmi		= (MINMAXINFO*) lParam;
	RECT		rect;

	// Width
	control	= GetDlgItem(dlg, IDC_CFG_FILE_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	= rect.left;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.x	-= rect.left;
	mmi->ptMinTrackSize.x	*= 2;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	+= rect.right;

	control	= GetDlgItem(dlg, IDC_CFG_FILE_AGAIN);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	-= rect.left;

	// Height
	control	= GetDlgItem(dlg, IDC_CFG_FILE_STATUS);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	= rect.bottom;

	control	= GetDlgItem(dlg, IDC_CFG_FILE_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	-= rect.bottom;
	mmi->ptMinTrackSize.y	+= rect.top;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.y	-= rect.top;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	+= 3 * (rect.bottom - rect.top);

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	HWND			control;
	config_save_t*	data;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data		= (config_save_t*) GET_WINDOW_USER_DATA(dlg);
	data->dlg	= dlg;

	set_win_title(dlg, data->device, FALSE, TRUE, "Configuration Save");

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	ctrl_menu_set_text(dlg, IDC_CFG_FILE_NAME, data->name);

	control	= GetDlgItem(dlg, IDC_CFG_FILE_LIST);
	SendMessage(control, WM_SETFONT, (WPARAM) data->device->font.fixed, 0);

	win_pos_restore(dlg, INI_SECTION_NAME, TRUE);
	_wm_size(dlg);
	_idc_cfg_file_again(dlg);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND dlg)
{
	config_save_t*	data	= (config_save_t*) GET_WINDOW_USER_DATA(dlg);

	static	align_t	list[]	=
	{
		// The outter frame.
		{// 0
		/* id		*/	IDC_CFG_FILE_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-1
		},
		{// 1
		/* id		*/	IDC_CFG_FILE_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_T,
		/* value	*/	0
		},
		{// 2
		/* id		*/	IDC_CFG_FILE_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R | ALIGN_B_B,
		/* value	*/	1
		},

		// The "File:" text.
		{// 3
		/* id		*/	IDC_CFG_FILE_TITLE,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L | ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 4
		/* id		*/	IDC_CFG_FILE_TITLE,
		/* ref		*/	IDC_CFG_FILE_FRAME_1,
		/* flags	*/	ALIGN_T_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The file name text.
		{// 5
		/* id		*/	IDC_CFG_FILE_NAME,
		/* ref		*/	IDC_CFG_FILE_TITLE,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	MARGIN
		},
		{// 6
		/* id		*/	IDC_CFG_FILE_NAME,
		/* ref		*/	IDC_CFG_FILE_TITLE,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 7
		/* id		*/	IDC_CFG_FILE_NAME,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},

		// The status text.
		{// 8
		/* id		*/	IDC_CFG_FILE_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 9
		/* id		*/	IDC_CFG_FILE_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 10
		/* id		*/	IDC_CFG_FILE_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_B_B | ALIGN_H,
		/* value	*/	- MARGIN
		},

		// The status frame.
		{// 11
		/* id		*/	IDC_CFG_FILE_FRAME_2,
		/* ref		*/	IDC_CFG_FILE_FRAME_1,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R | ALIGN_B_B,
		/* value	*/	0
		},
		{// 12
		/* id		*/	IDC_CFG_FILE_FRAME_2,
		/* ref		*/	IDC_CFG_FILE_STATUS,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	- MARGIN
		},

		// The ABORT button.
		{// 13
		/* id		*/	IDC_CFG_FILE_ABORT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_HC_C | ALIGN_W,
		/* value	*/	0
		},
		{// 14
		/* id		*/	IDC_CFG_FILE_ABORT,
		/* ref		*/	IDC_CFG_FILE_FRAME_2,
		/* flags	*/	ALIGN_B_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The AGAIN button.
		{// 15
		/* id		*/	IDC_CFG_FILE_AGAIN,
		/* ref		*/	IDC_CFG_FILE_ABORT,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 16
		/* id		*/	IDC_CFG_FILE_AGAIN,
		/* ref		*/	IDC_CFG_FILE_ABORT,
		/* flags	*/	ALIGN_R_L | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The CLOSE button.
		{// 17
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_CFG_FILE_ABORT,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 18
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_CFG_FILE_ABORT,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The list box.
		{// 19
		/* id		*/	IDC_CFG_FILE_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 20
		/* id		*/	IDC_CFG_FILE_LIST,
		/* ref		*/	IDC_CFG_FILE_TITLE,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	-999	// see below
		},
		{// 21
		/* id		*/	IDC_CFG_FILE_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 22
		/* id		*/	IDC_CFG_FILE_LIST,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	-999	// see below
		},

		{ 0, 0, 0, 0 }
	};

	list[3].value	= data->device->margin;
	list[4].value	= data->device->margin;
	list[7].value	= - data->device->margin;
	list[8].value	= data->device->margin;
	list[9].value	= - data->device->margin;
	list[14].value	= - data->device->margin;
	list[16].value	= - data->device->margin;
	list[18].value	= data->device->margin;
	list[19].value	= data->device->margin;
	list[20].value	= data->device->margin;
	list[21].value	= - data->device->margin;
	list[22].value	= - data->device->margin;

	align_control_list(dlg, list);
	InvalidateRect(dlg, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_GETMINMAXINFO:

			ret	= _wm_getminmaxinfo(dlg, lParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SIZE:

			ret	= _wm_size(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_CFG_FILE_SAVE, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_CFG_FILE_SAVE, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	config_save_t	data;
	int				i;
	OPENFILENAME	ofn;

	strcpy(data.name, "hpdi32.cfg");
	GetCurrentDirectory(sizeof(data.dir), data.dir);

	// Initialize OPENFILENAME
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= dlg;
	ofn.lpstrFilter		= "Configuration Files (*.cfg)\0*.cfg\0"
							"All (*.*)\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= data.name;
	ofn.nMaxFile		= sizeof(data.name);
	ofn.lpstrInitialDir	= data.dir;
	ofn.lpstrTitle		= "Configuration Save";
	ofn.Flags			= OFN_PATHMUSTEXIST
						| OFN_OVERWRITEPROMPT
						| OFN_HIDEREADONLY;
	i					= GetSaveFileName(&ofn);

	if (i)
	{
		main_close_all(dlg, device);
		data.abort	= FALSE;
		data.device	= device;
		config_list_retrieve(&data.list, &data.size);
		DialogBoxParam(	device->instance,
						(LPCTSTR) IDD_CFG_FILE,
						dlg,
						(DLGPROC) _dlg_proc,
						(LPARAM) &data);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	config_save_service
*
*	Purpose:
*
*		Implement the common service entry point for the Configuration Save
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT config_save_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


