// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/config_wizard.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement services for the Configuration Wizard.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "config.h"
#include "c_code.h"
#include "cable.h"
#include "copy.h"
#include "fifo.h"
#include "gui.h"
#include "io.h"
#include "misc.h"
#include "resource.h"
#include "rx.h"
#include "tx.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"configuration:wizard"



/* typedefs *****************************************************************/

typedef struct
{
	int			changes;		// Are there pending changes?
	HWND		control;		// Used to track if changes are pending.
	device_t*	device;
	HWND		dlg;
	int			edit_height;
	int			edit_width;
	int			icon_height;
	int			icon_width;
	int			ignore;			// Used to control display value updates.
	int			radio_height;
	U32			value;			// Value being tracked for current parameter.
	int			x_spacing;
	int			y_spacing;

	const config_item_t*	list;
	size_t					size;
	int						index;
	int						parm_index;
} config_t;



/* prototypes	**************************************************************/

static void _idcancel(HWND dlg);



/* variables *****************************************************************/

static const int	_radio_list[4]	=
{
	IDC_CFG_WIZ_DATA_RADIO_1,
	IDC_CFG_WIZ_DATA_RADIO_2,
	IDC_CFG_WIZ_DATA_RADIO_3,
	IDC_CFG_WIZ_DATA_RADIO_4
};



/*****************************************************************************
*
*	Function:	_hide_control
*
*	Purpose:
*
*		Make a control so it isn't seen in the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _hide_control(HWND dlg, int id)
{
	HWND	control;

	control	= GetDlgItem(dlg, id);
	MoveWindow(control, -1, -1, -1, -1, TRUE);
	EnableWindow(control, FALSE);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Retrieve the current parameter's value from the device.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _device_read(config_t* data)
{
	const parm_entry_t*			entry;
	const config_parm_list_t*	parm;
	U32							status;

	parm	= (const config_parm_list_t*) data->list[data->index].vp;
	entry	= parm->list->list[data->parm_index];
	status	= hpdi32_config(		data->device->handle,
									entry->parm,
									entry->which | parm->list->which,
									GSC_NO_CHANGE,
									&data->value);
	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(config_t* data)
{
	HCURSOR						cursor;
	const parm_entry_t*			entry;
	const config_parm_list_t*	parm;
	U32							status;

	cursor	= SetCursor(data->device->cursor.wait);
	parm	= (const config_parm_list_t*) data->list[data->index].vp;
	entry	= parm->list->list[data->parm_index];
	status	= hpdi32_config(data->device->handle,
							entry->parm,
							entry->which | parm->list->which,
							data->value,
							NULL);
	SetCursor(cursor);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, config_t* data)
{
	const parm_entry_t*			entry;
	int							i;
	const config_parm_list_t*	parm;
	static ctrl_id_u32_t		r_list[4];
	U32							status	= GSC_SUCCESS;

	parm	= (const config_parm_list_t*) data->list[data->index].vp;
	entry	= parm->list->list[data->parm_index];

	switch (entry->values->type)
	{
		default:

			break;

		case VALUE_TYPE_LIST:

			r_list[0].ctrl_id	= IDC_CFG_WIZ_DATA_RADIO_1;
			r_list[1].ctrl_id	= IDC_CFG_WIZ_DATA_RADIO_2;
			r_list[2].ctrl_id	= IDC_CFG_WIZ_DATA_RADIO_3;
			r_list[3].ctrl_id	= IDC_CFG_WIZ_DATA_RADIO_4;

			for (i = 0; i < (int) entry->values->u.list->size; i++)
				r_list[i].u32	= entry->values->u.list->list[i].value;

			data->value	= ctrl_radio_2_u32(dlg, r_list, (size_t) i, 0);
			break;

		case VALUE_TYPE_RANGE:

			status	= ctrl_text_2_u32(	dlg,
										IDC_CFG_WIZ_DATA_EDIT_INPUT,
										entry->values->u.range->k,
										&data->value);
			break;
	}

	return(status);
}


/*****************************************************************************
*
*	Function:	_show_parm_value_list
*
*	Purpose:
*
*		Display a list of values using the radio buttons.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		entry		The data structure for the parameter being accessed.
*
*		init		Initialize the controls and such?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _show_parm_value_list(
	HWND				dlg,
	config_t*			data,
	const parm_entry_t*	entry,
	int					init)
{
	HWND				control;
	int					i;
	ctrl_id_u32_t		list[4];
	RECT				rect;
	U32					stat;
	U32					status	= GSC_SUCCESS;
	const value_list_t*	values	= entry->values->u.list;
	int					w		= 0;
	int					x		= 0;
	int					y		= 0;

	if (init)
	{
		control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_DESC);
		GetWindowRect(control, &rect);
		ScreenToClient(dlg, (POINT*) &rect.left);
		ScreenToClient(dlg, (POINT*) &rect.right);
		x	= rect.left;

		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RECT);
		GetWindowRect(control, &rect);
		ScreenToClient(dlg, (POINT*) &rect.left);
		ScreenToClient(dlg, (POINT*) &rect.right);
		y	= ((rect.bottom - rect.top) / 2);
		y	-= (4 * data->radio_height) + (3 * data->y_spacing);
		y	= rect.top + (y / 2);
		w	= rect.right - x;
	}

	memset(list, 0, sizeof(list));

	for (i = 0; i < 4; i++)
	{
		if (i >= (int) values->size)
		{
			ctrl_menu_set(dlg, _radio_list[i], CTRL_BM_UNCHECK);
			continue;
		}

		list[i].ctrl_id	= _radio_list[i];
		list[i].u32		= entry->values->u.list->list[i].value;

		if (init)
		{
			control	= GetDlgItem(dlg, _radio_list[i]);
			EnableWindow(control, TRUE);
			SetWindowText(control, values->list[i].user_text);
			MoveWindow(control, x, y, w, data->radio_height, TRUE);
			y	+= data->radio_height + data->y_spacing;
		}
	}

	if (entry->cfg_wiz)
	{
		stat	= (entry->cfg_wiz)(dlg, data->device, entry, data->value);
		status	= (status == GSC_SUCCESS) ? stat : status;
	}

	ctrl_u32_2_radio(dlg, data->value, list, values->size);
	return(status);
}



/*****************************************************************************
*
*	Function:	_show_parm_value_range
*
*	Purpose:
*
*		Display a parameter value that should be within a range.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		entry		The data structure for the parameter being accessed.
*
*		init		Initialize the controls and such?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _show_parm_value_range(
	HWND				dlg,
	config_t*			data,
	const parm_entry_t*	entry,
	int					init)
{
	char					buf[64];
	HWND					control;
	RECT					rect;
	U32						status	= GSC_SUCCESS;
	const value_range_t*	values	= entry->values->u.range;
	int						x;
	int						y;

	if (init)
	{
		control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_DESC);
		GetWindowRect(control, &rect);
		ScreenToClient(dlg, (POINT*) &rect.left);
		ScreenToClient(dlg, (POINT*) &rect.right);
		x	= rect.left;

		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RECT);
		GetWindowRect(control, &rect);
		ScreenToClient(dlg, (POINT*) &rect.left);
		ScreenToClient(dlg, (POINT*) &rect.right);
		y	= x + rect.top;

		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT_INPUT);
		EnableWindow(control, TRUE);
		MoveWindow(control, x, y, data->edit_width, data->edit_height, TRUE);

		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT_STATUS);
		EnableWindow(control, TRUE);
		MoveWindow(	control,
					x + data->edit_width + rect.left,
					y,
					rect.right - (x + data->edit_width + rect.left),
					data->edit_height,
					TRUE);
	}


	if (data->ignore == FALSE)
	{
		data->ignore	= TRUE;
		format_u32(data->value, values->k, buf);
		ctrl_menu_set_text(dlg, IDC_CFG_WIZ_DATA_EDIT_INPUT, buf);
		data->ignore	= FALSE;
	}

	sprintf(buf,
			"(Valid Range = 0x%lX - 0x%lX, 1K = %d)",
			(long) values->min,
			(long) values->max,
			(int) values->k);
	ctrl_menu_set_text(dlg, IDC_CFG_WIZ_DATA_EDIT_STATUS, buf);

	if (entry->cfg_wiz)
		status	= (entry->cfg_wiz)(dlg, data->device, entry, data->value);

	return(status);
}



/*****************************************************************************
*
*	Function:	_show_type_parm_list
*
*	Purpose:
*
*		Update the window for a TYPE_PARM_LIST display item.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*		init	Initialize the controls and such?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _show_type_parm_list(HWND dlg, config_t* data, int init)
{
	char						buf[1024];
	HWND						control;
	const parm_entry_t*			entry;
	const config_parm_list_t*	parm;
	RECT						rect;
	U32							status	= GSC_SUCCESS;

	parm	= (const config_parm_list_t*) data->list[data->index].vp;
	entry	= parm->list->list[data->parm_index];

	if (init)
	{
		strcpy(buf, parm->title);
		strcat(buf, ": ");

		if (parm->prefix)
		{
			strcat(buf, parm->prefix);
			strcat(buf, " ");
		}

		strcat(buf, entry->title);
		ctrl_menu_set_text(dlg, IDC_CFG_WIZ_TITLE_DESC, buf);

		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RECT);
		GetWindowRect(control, &rect);
		ScreenToClient(dlg, (POINT*) &rect.left);
		ScreenToClient(dlg, (POINT*) &rect.right);

		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT);
		EnableWindow(control, TRUE);

		if (entry->desc)
			SetWindowText(control, entry->desc);
		else
			SetWindowText(control, "");

		MoveWindow(	control,
					rect.left,
					rect.top + ((rect.bottom - rect.top) / 2),
					rect.right - rect.left,
					(rect.bottom - rect.top) / 2,
					TRUE);
		ctrl_menu_set(dlg, IDC_APPLY, CTRL_ENABLE);
	}

	switch (entry->values->type)
	{
		default:

			break;

		case VALUE_TYPE_LIST:

			status	= _show_parm_value_list(dlg, data, entry, init);
			break;

		case VALUE_TYPE_RANGE:

			status	= _show_parm_value_range(dlg, data, entry, init);
			break;
	}

	strcpy(buf, "C Code: status = ");
	c_code_format_entry(data->device,
						parm->list,
						data->parm_index,
						TRUE,
						data->value,
						buf + strlen(buf));
	ctrl_menu_set_text(dlg, IDC_CFG_WIZ_STATUS_TEXT, buf);
	return(status);
}



/*****************************************************************************
*
*	Function:	_show_type_progress
*
*	Purpose:
*
*		Update the window for a TYPE_PROGRESS display item.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _show_type_progress(HWND dlg, config_t* data)
{
	HWND						control;
	int							ddy;
	int							dx;
	int							dy;
	RECT						frame;
	int							h;
	HICON						icon;
	int							margin;
	const config_progress_t*	ptr;
	RECT						rect;
	int							w;
	int							x1;
	int							x2;
	int							y;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_RECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	dy	= rect.bottom;
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RECT);
	GetWindowRect(control, &frame);
	ScreenToClient(dlg, (POINT*) &frame.left);
	ScreenToClient(dlg, (POINT*) &frame.right);
	dy		= frame.top - dy;
	dx		= frame.left;
	margin	= ((frame.bottom - frame.top) - (3 * dy) - (4 * data->icon_height)) / 2;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_STATUS_TEXT);
	GetWindowRect(control, &rect);
	h			= rect.bottom - rect.top;
	ddy			= (data->icon_height - h) / 2;
	w			= (frame.right - frame.left)
				- (2 * (margin - dx))
				- dx
				- data->icon_width;
	ptr			= (const config_progress_t*) data->list[data->index].vp;

	x1		= frame.left + margin;
	x2		= x1 + data->icon_width;
	y		= frame.top + margin;
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_ICON_1);
	MoveWindow(control, x1, y, data->icon_width, data->icon_height, TRUE);
	icon	= LoadIcon(data->device->instance, MAKEINTRESOURCE(ptr->misc));
	SendMessage(control, STM_SETIMAGE, IMAGE_ICON, (LPARAM)(UINT) icon);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_TEXT_1);
	EnableWindow(control, TRUE);
	MoveWindow(control, x2, y + ddy, w, h, TRUE);
	SetWindowText(control, "Miscellaneous Parameters");

	y		+= data->icon_height + dy;
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_ICON_2);
	MoveWindow(control, x1, y, data->icon_width, data->icon_height, TRUE);
	icon	= LoadIcon(data->device->instance, MAKEINTRESOURCE(ptr->cable));
	SendMessage(control, STM_SETIMAGE, IMAGE_ICON, (LPARAM)(UINT) icon);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_TEXT_2);
	EnableWindow(control, TRUE);
	MoveWindow(control, x2, y + ddy, w, h, TRUE);
	SetWindowText(control, "Cable Parameters");

	y		+= data->icon_height + dy;
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_ICON_3);
	MoveWindow(control, x1, y, data->icon_width, data->icon_height, TRUE);
	icon	= LoadIcon(data->device->instance, MAKEINTRESOURCE(ptr->tx));
	SendMessage(control, STM_SETIMAGE, IMAGE_ICON, (LPARAM)(UINT) icon);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_TEXT_3);
	EnableWindow(control, TRUE);
	MoveWindow(control, x2, y + ddy, w, h, TRUE);
	SetWindowText(control, "Transmit Related Parameters");

	y		+= data->icon_height + dy;
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_ICON_4);
	MoveWindow(control, x1, y, data->icon_width, data->icon_height, TRUE);
	icon	= LoadIcon(data->device->instance, MAKEINTRESOURCE(ptr->rx));
	SendMessage(control, STM_SETIMAGE, IMAGE_ICON, (LPARAM)(UINT) icon);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_TEXT_4);
	EnableWindow(control, TRUE);
	MoveWindow(control, x2, y + ddy, w, h, TRUE);
	SetWindowText(control, "Receive Related Parameters");
}



/*****************************************************************************
*
*	Function:	_show_type_text
*
*	Purpose:
*
*		Update the window for a TYPE_TEXT display item.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _show_type_text(HWND dlg, config_t* data)
{
	HWND	control;
	RECT	rect;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_TEXT);
	EnableWindow(control, TRUE);
	SetWindowText(control, (char*) data->list[data->index].vp);
	MoveWindow(	control,
				rect.left,
				rect.top,
				rect.right - rect.left,
				rect.bottom - rect.top,
				TRUE);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window's controls per the state of the device
*		in use, if any.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*		init	Initialize the controls and such?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, config_t* data, int init)
{
	char	buf[128];
	int		i		= data->index;
	U32		status	= GSC_SUCCESS;

	if (init)
	{
		InvalidateRect(dlg, NULL, TRUE);
		UpdateWindow(dlg);
		data->changes	= FALSE;
		data->control	= NULL;
		data->ignore	= FALSE;
	}

	strcpy(buf, "Configuration Wizard");
	strcat(buf, data->changes ? " *" : "");
	set_win_title(dlg, data->device, FALSE, TRUE, buf);

	if (init)
	{
		ctrl_menu_set_text(dlg, IDC_CFG_WIZ_TITLE_TEXT, data->list[i].title);
		ctrl_menu_set_text(dlg, IDC_CFG_WIZ_TITLE_DESC, data->list[i].desc);
		ctrl_menu_set(dlg, IDC_CFG_WIZ_STATUS_TEXT, CTRL_NULL);
		ctrl_menu_set(dlg, IDC_APPLY, CTRL_DISABLE);

		if (i <= 0)
		{
			ctrl_menu_set(dlg, IDC_CFG_WIZ_BACK, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CFG_WIZ_NEXT, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_CFG_WIZ_BACK, CTRL_ENABLE);
		}

		if (i >= (int) (data->size - 1))
		{
			ctrl_menu_set(dlg, IDC_CFG_WIZ_NEXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		}
		else
		{
			ctrl_menu_set(dlg, IDC_CFG_WIZ_NEXT, CTRL_ENABLE);
		}

		if (i < (int) (data->size - 1))
			ctrl_menu_set_text(dlg, IDCANCEL, "Cancel");
		else
			ctrl_menu_set_text(dlg, IDCANCEL, "Finish");

		_hide_control(dlg, IDC_CFG_WIZ_DATA_TEXT);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_ICON_1);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_TEXT_1);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_ICON_2);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_TEXT_2);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_ICON_3);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_TEXT_3);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_ICON_4);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_TEXT_4);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_EDIT);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_RADIO_1);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_RADIO_2);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_RADIO_3);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_RADIO_4);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_EDIT_INPUT);
		_hide_control(dlg, IDC_CFG_WIZ_DATA_EDIT_STATUS);

		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_1, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_2, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_3, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_4, CTRL_DISABLE);
	}

	switch (data->list[i].type)
	{
		default:

			break;

		case TYPE_PARM_LIST:

			status	= _show_type_parm_list(dlg, data, init);
			break;

		case TYPE_PROGRESS:

			_show_type_progress(dlg, data);
			break;

		case TYPE_TEXT:

			_show_type_text(dlg, data);
			break;
	}

	if (init)
		InvalidateRect(dlg, NULL, FALSE);

	error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg)
{
	config_t*	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);
	U32			stat;
	U32			status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	_refresh_window(dlg, data, FALSE);
	error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
	main_refresh(data->device, NULL);
}



/*****************************************************************************
*
*	Function:	_idc_cfg_wiz_back
*
*	Purpose:
*
*		Respond to the BACK button.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cfg_wiz_back(HWND dlg)
{
	config_t*					data;
	const config_parm_list_t*	parm;
	U32							status	= GSC_SUCCESS;

	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);

	if (data->list[data->index].type == TYPE_PARM_LIST)
		_idc_apply(dlg);

	if ((data->list[data->index].type != TYPE_PARM_LIST) ||
		(data->parm_index == 0))
	{
		data->index--;

		if (data->list[data->index].type != TYPE_PARM_LIST)
		{
			data->parm_index	= 0;
		}
		else
		{
			parm	= (const config_parm_list_t*) data->list[data->index].vp;
			data->parm_index	= (int) parm->list->size - 1;
		}

		if (data->list[data->index].type == TYPE_PARM_LIST)
			status	= _device_read(data);

		_refresh_window(dlg, data, TRUE);
	}
	else
	{
		data->parm_index--;

		if (data->list[data->index].type == TYPE_PARM_LIST)
			status	= _device_read(data);

		_refresh_window(dlg, data, TRUE);
	}

	error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
}



/*****************************************************************************
*
*	Function:	_idc_cfg_wiz_data_edit_input
*
*	Purpose:
*
*		Respond to activity in the API Buffer Size edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cfg_wiz_data_edit_input(HWND dlg, WPARAM wParam)
{
	int			code	= HIWORD(wParam);
	config_t*	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);
	U32			status;

	if ((code == EN_CHANGE) && (data->ignore == FALSE))
	{
		data->changes	= TRUE;
		data->ignore	= TRUE;
		status			= _dialog_read(dlg, data);
		_refresh_window(dlg, data, FALSE);
		data->ignore	= FALSE;
		error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
	}
}



/*****************************************************************************
*
*	Function:	_idc_cfg_wiz_data_radio_x
*
*	Purpose:
*
*		Respond to activity on the radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	The message's WORD parameter.
*
*		lParam	The message's LONG parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cfg_wiz_data_radio_x(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	int			code;
	HWND		control;
	config_t*	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);
	LRESULT		state;
	U32			status;

	code	= HIWORD(wParam);
	control	= (HWND) lParam;

	if (code == BN_SETFOCUS)
	{
		state	= SendMessage(control, BM_GETCHECK, 0, 0);

		if (state == BST_UNCHECKED)
			data->control	= control;
		else
			data->control	= NULL;
	}
	else if ((code == BN_CLICKED) && (data->control == control))
	{
		data->changes	= TRUE;
		status			= _dialog_read(dlg, data);
		_refresh_window(dlg, data, FALSE);
		error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
	}
}



/*****************************************************************************
*
*	Function:	_idc_cfg_wiz_next
*
*	Purpose:
*
*		Respond to the NEXT button.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_cfg_wiz_next(HWND dlg)
{
	config_t*					data;
	const config_parm_list_t*	parm;
	U32							status	= GSC_SUCCESS;

	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);

	if (data->list[data->index].type == TYPE_PARM_LIST)
		_idc_apply(dlg);

	if (data->index == (int) (data->size - 1))
	{
		_idcancel(dlg);
	}
	else if (data->list[data->index].type != TYPE_PARM_LIST)
	{
		data->index++;
		data->parm_index	= 0;

		if (data->list[data->index].type == TYPE_PARM_LIST)
			status	= _device_read(data);

		_refresh_window(dlg, data, TRUE);
	}
	else
	{
		parm	= (const config_parm_list_t*) data->list[data->index].vp;

		if (data->parm_index >= (int) (parm->list->size - 1))
		{
			data->index++;
			data->parm_index	= 0;
		}
		else
		{
			data->parm_index++;
		}


		if (data->list[data->index].type == TYPE_PARM_LIST)
			status	= _device_read(data);

		_refresh_window(dlg, data, TRUE);
	}

	error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	#define	_RADIO	(COPY_EOL | COPY_VISIBLE | COPY_DISABLED | COPY_BM_GETCHECK)
	#define	_TEXT	(COPY_EOL | COPY_VISIBLE | COPY_WRAP | 70)

	static const copy_t	list[]	=
	{
		/* id							flags						*/
		{ -1,							COPY_EOL | COPY_UNDERLINE	},

		{ IDC_CFG_WIZ_TITLE_TEXT,		COPY_EOL					},
		{ IDC_CFG_WIZ_TITLE_DESC,		COPY_EOL					},

		{ 0,							COPY_UNDERLINE				},
		{ 0,							COPY_EOL					},

		{ IDC_CFG_WIZ_DATA_TEXT,		_TEXT						},
		{ IDC_CFG_WIZ_DATA_TEXT_1,		COPY_EOL | COPY_VISIBLE		},
		{ IDC_CFG_WIZ_DATA_TEXT_2,		COPY_EOL | COPY_VISIBLE		},
		{ IDC_CFG_WIZ_DATA_TEXT_3,		COPY_EOL | COPY_VISIBLE		},
		{ IDC_CFG_WIZ_DATA_TEXT_4,		COPY_EOL | COPY_VISIBLE		},
		{ IDC_CFG_WIZ_DATA_RADIO_1,		_RADIO						},
		{ IDC_CFG_WIZ_DATA_RADIO_2,		_RADIO						},
		{ IDC_CFG_WIZ_DATA_RADIO_3,		_RADIO						},
		{ IDC_CFG_WIZ_DATA_RADIO_4,		_RADIO						},

		{ IDC_CFG_WIZ_DATA_EDIT_INPUT,	COPY_VISIBLE							},
		{ IDC_CFG_WIZ_DATA_EDIT_STATUS,	COPY_EOL | COPY_VISIBLE | COPY_INDENT	},

		{ IDC_CFG_WIZ_DATA_EDIT,		_TEXT | COPY_PREFEED		},

		{ 0,							COPY_UNDERLINE				},
		{ 0,							COPY_EOL					},

		{ IDC_CFG_WIZ_STATUS_TEXT,		COPY_EOL					},

		{ 0,							COPY_UNDERLINE				},
		{ 0,							COPY_EOL					},

		{ IDC_APPLY,					COPY_EOL | COPY_DISABLED	},
		{ IDC_CFG_WIZ_BACK,				COPY_EOL | COPY_DISABLED	},
		{ IDC_CFG_WIZ_NEXT,				COPY_EOL | COPY_DISABLED	},
		{ IDCANCEL,						COPY_EOL | COPY_DISABLED	}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));

	#undef _RADIO
	#undef _TEXT
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	config_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		data->dlg	= NULL;
		DeleteObject(data->device->paint.gray_dk.pen);
	}

	win_pos_save(dlg, SECTION_NAME);
	EndDialog(dlg, 0);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	int		id	= LOWORD(wParam);
	LRESULT	ret	= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg);
			break;

		case IDC_CFG_WIZ_BACK:

			_idc_cfg_wiz_back(dlg);
			break;

		case IDC_CFG_WIZ_DATA_EDIT_INPUT:

			_idc_cfg_wiz_data_edit_input(dlg, wParam);
			break;

		case IDC_CFG_WIZ_DATA_RADIO_1:
		case IDC_CFG_WIZ_DATA_RADIO_2:
		case IDC_CFG_WIZ_DATA_RADIO_3:
		case IDC_CFG_WIZ_DATA_RADIO_4:

			_idc_cfg_wiz_data_radio_x(dlg, wParam, lParam);
			break;

		case IDC_CFG_WIZ_NEXT:

			_idc_cfg_wiz_next(dlg);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_ctlcolorstatic
*
*	Purpose:
*
*		Process a WM_CTLCOLORSTATIC message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		The brush handle to use for the background.
*
*****************************************************************************/

static LRESULT _wm_ctlcolorstatic(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	HWND		control	= (HWND) lParam;
	config_t*	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);
	HDC			dc	= (HDC) wParam;
	int			id		= GetWindowLong(control, GWL_ID);
	LRESULT		ret		= 0;

	if ((id == IDC_CFG_WIZ_TITLE_TEXT) ||
		(id == IDC_CFG_WIZ_TITLE_DESC))
	{
		ret	= (LRESULT) data->device->paint.btnhighlt.brush;
		SetBkColor(dc, data->device->paint.btnhighlt.color);
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	HWND		control;
	config_t*	data;
	RECT		rect;
	U32			status	= GSC_SUCCESS;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_TEXT);
	SendMessage(control, WM_SETFONT, (WPARAM) data->device->font.defalt_bll, 0);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_STATUS_TEXT);
	SendMessage(control, WM_SETFONT, (WPARAM) data->device->font.fixed_l, 0);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_ICON_1);
	GetWindowRect(control, &rect);
	data->icon_height	= rect.bottom - rect.top;
	data->icon_width	= rect.right - rect.left;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RADIO_1);
	GetWindowRect(control, &rect);
	data->radio_height	= rect.bottom - rect.top;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT_INPUT);
	GetWindowRect(control, &rect);
	data->edit_height	= rect.bottom - rect.top;
	data->edit_width	= rect.right - rect.left;

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_RECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	data->y_spacing	= rect.bottom;
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_RECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	data->x_spacing	= rect.left;
	data->y_spacing	= rect.top - data->y_spacing;

	win_pos_restore(dlg, SECTION_NAME, FALSE);

	_refresh_window(dlg, data, TRUE);
	error_status(dlg, IDC_CFG_WIZ_STATUS_TEXT, status);
	SetFocus(dlg);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_paint
*
*	Purpose:
*
*		Process a WM_PAINT message.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The data we're to display.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_paint(HWND dlg)
{
	HBRUSH		brush_old;
	HWND		control;
	config_t*	data	= (config_t*) GET_WINDOW_USER_DATA(dlg);
	HDC			dc;
	HPEN		pen_old;
	RECT		rect;

	dc	= GetDC(dlg);

	// Paint the title area.
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_RECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	brush_old	= SelectObject(dc, data->device->paint.btnhighlt.brush);
	pen_old		= SelectObject(dc, data->device->paint.black.pen);

	Rectangle(	dc,
				rect.left - 1,
				rect.top,
				rect.right + 1,
				rect.bottom);

	SelectObject(dc, data->device->paint.gray_dk.pen);
	MoveToEx(dc, rect.left, rect.bottom - 2, NULL);
	LineTo(dc, rect.right, rect.bottom - 2);
	SelectObject(dc, data->device->paint.btnhighlt.pen);
	MoveToEx(dc, rect.left, rect.bottom - 1, NULL);
	LineTo(dc, rect.right, rect.bottom - 1);

	// Paint the status area.
	control	= GetDlgItem(dlg, IDC_CFG_WIZ_STATUS_RECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	SelectObject(dc, data->device->paint.black.pen);

	SelectObject(dc, data->device->paint.gray_dk.pen);
	MoveToEx(dc, rect.left, rect.top, NULL);
	LineTo(dc, rect.right, rect.top);
	MoveToEx(dc, rect.left, rect.bottom - 2, NULL);
	LineTo(dc, rect.right, rect.bottom - 2);

	SelectObject(dc, data->device->paint.btnhighlt.pen);
	MoveToEx(dc, rect.left, rect.top + 1, NULL);
	LineTo(dc, rect.right, rect.top + 1);
	MoveToEx(dc, rect.left, rect.bottom - 1, NULL);
	LineTo(dc, rect.right, rect.bottom - 1);

	// We're done.
	SelectObject(dc, brush_old);
	SelectObject(dc, pen_old);
	ReleaseDC(dlg, dc);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_TEXT);
	InvalidateRect(control, NULL, TRUE);
	UpdateWindow(control);

	control	= GetDlgItem(dlg, IDC_CFG_WIZ_TITLE_DESC);
	InvalidateRect(control, NULL, TRUE);
	UpdateWindow(control);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_CTLCOLORSTATIC:

			ret	= _wm_ctlcolorstatic(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_PAINT:

			ret	= _wm_paint(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_CFG_WIZ, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_CFG_WIZ, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	config_t	data;

	main_close_all(dlg, device);
	data.device		= device;
	config_list_retrieve(&data.list, &data.size);
	data.index		= 0;
	data.parm_index	= 0;
	DialogBoxParam(	device->instance,
					(LPCTSTR) IDD_CFG_WIZ,
					dlg,
					(DLGPROC) _dlg_proc,
					(LPARAM) &data);
	return(0);
}



/*****************************************************************************
*
*	Function:	config_wizard_service
*
*	Purpose:
*
*		Implement the common service entry point for the Configuration Wizard
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT config_wizard_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


