// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/device_id.c $
// $Rev: 31452 $
// $Date: 2014-11-13 09:58:59 -0600 (Thu, 13 Nov 2014) $

/*****************************************************************************
*
*	Implement the Device ID dialog.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "device_id.h"
#include "align.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	_32_BIT_DEVICE	"This is an HPDI32 with a 32-bit PCI interface."
#define	_64_BIT_DEVICE	"This is an HPDI32 with a 64-bit PCI interface."
#define	SECTION_NAME	"device id"



/* typedefs	*****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
	char		status_local[128];
} _id_t;



/* prototypes	*************************************************************/

static	LRESULT	_wm_size(HWND dlg);



/*****************************************************************************
*
*	Function:	_add_entry
*
*	Purpose:
*
*		Add an entry to the list box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		label		The label for the entry
*
*		info		The information for the entry.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _add_entry(HWND dlg, const char* label, const char* info)
{
	char	buf[1024];
	HWND	control		= GetDlgItem(dlg, IDC_DEVICE_ID_LIST);

	strcpy(buf, label);
	strcat(buf, ":                              ");
	buf[24]	= 0;
	strcat(buf, info);
	SendMessage(control, LB_ADDSTRING, 0, (LPARAM) buf);
}



/*****************************************************************************
*
*	Function:	_jumper_id
*
*	Purpose:
*
*		Report the Board Jumper ID.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _jumper_id(HWND dlg, _id_t* data)
{
	char	buf[128];
	U32		id;
	U32		stat;
	U32		status	= GSC_SUCCESS;
	U32		uj;

	stat	= HPDI32_MISC_FEATURES__USER_JUMPERS(data->device->handle, &uj);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_MISC_BOARD_JUMPERS__GET(data->device->handle, &id);
	status	= (status == GSC_SUCCESS) ? stat : status;
	sprintf(buf, "0x%lX", (long) id);
	sprintf(buf + strlen(buf), " (#1 is %s", (id & 0x2) ? "On" : "Off");
	sprintf(buf + strlen(buf), ", #0 is %s)", (id & 0x1) ? "On" : "Off");

	if (uj == HPDI32_MISC_FEATURES_ABSENT)
		strcpy(buf, "(The jumpers are not supported on this board.)");

	_add_entry(dlg, "User Jumper ID", buf);

	return(status);
}



/*****************************************************************************
*
*	Function:	_pci_id
*
*	Purpose:
*
*		Report the PCI identification.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		did			The Device ID is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _pci_id(HWND dlg, _id_t* data, U32* did)
{
	char	buf[128];
	U32		sid;
	U32		stat;
	U32		status	= GSC_SUCCESS;
	U32		svid;
	U32		vid;

	// Vendor ID
	stat	= hpdi32_reg_read(data->device->handle, GSC_PCI_9080_VIDR, &vid);
	status	= (status == GSC_SUCCESS) ? stat : status;
	sprintf(buf, "0x%04lX", (long) vid);

	if (vid == HPDI32_VENDOR_ID)
		strcat(buf, " (PLX)");
	else
		strcat(buf, " (NOT RECOGNIZED)");

	_add_entry(dlg, "Vendor ID", buf);

	// Device ID
	stat	= hpdi32_reg_read(data->device->handle, GSC_PCI_9080_DIDR, did);
	status	= (status == GSC_SUCCESS) ? stat : status;
	sprintf(buf, "0x%04lX", (long) did[0]);

	if (did[0] == HPDI32_DEVICE_ID_32)
	{
		strcat(buf, " (PCI9080)");
		strcpy(data->status_local, _32_BIT_DEVICE);
	}
	else if (did[0] == HPDI32_DEVICE_ID_64)
	{
		strcat(buf, " (PCI9656)");
		strcpy(data->status_local, _64_BIT_DEVICE);
	}
	else
	{
		strcat(buf, " (NOT RECOGNIZED)");
	}

	_add_entry(dlg, "Device ID", buf);

	// Subsystem Vendor ID
	stat	= hpdi32_reg_read(data->device->handle, GSC_PCI_9080_SVID, &svid);
	status	= (status == GSC_SUCCESS) ? stat : status;
	sprintf(buf, "0x%04lX", (long) svid);

	if (svid == HPDI32_SUBVENDOR_ID)
		strcat(buf, " (PLX)");
	else
		strcat(buf, " (NOT RECOGNIZED)");

	_add_entry(dlg, "Subsystem Vendor ID", buf);

	// Subsystem ID
	stat	= hpdi32_reg_read(data->device->handle, GSC_PCI_9080_SID, &sid);
	status	= (status == GSC_SUCCESS) ? stat : status;
	sprintf(buf, "0x%04lX", (long) sid);

	if (sid == HPDI32_SUBSYSTEM_ID_32)
		strcat(buf, " (HPDI32 (32-bit PCI))");
	else if (sid == HPDI32_SUBSYSTEM_ID_64)
		strcat(buf, " (HPDI32 (64-bit PCI))");
	else
		strcat(buf, " (NOT RECOGNIZED)");

	_add_entry(dlg, "Subsystem ID", buf);

	return(status);
}



/*****************************************************************************
*
*	Function:	_frr_original_id
*
*	Purpose:
*
*		Report the Firmware Revision Register identification for one that has
*		the original register format.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		did			The PCI Device ID register value.
*
*		frr			The value from the Firmware Revision Register.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _frr_original_id(HWND dlg, U32 did, U32 frr)
{
	char	buf[128];
	long	fr;
	long	pcb;
	long	res;
	long	sub;

	// Firmware Revision
	fr	= HPDI32_FRR_FIRMWARE_DECODE(frr);
	sprintf(buf, "0x%02lX", fr);
	_add_entry(dlg, "  Firmware Revision", buf);

	// PCB Revision
	pcb	= HPDI32_FRR_PCB_DECODE(frr);
	sprintf(buf, "0x%02lX", pcb);

	if (pcb == 0)
		strcat(buf, " (no revisions)");
	else
		sprintf(buf + strlen(buf), " (Rev. %c)", (char) ('A' + pcb - 1));

	_add_entry(dlg, "  PCB Revision", buf);

	// Sub ID
	sub	= HPDI32_FRR_SUB_ID_DECODE(frr);
	sprintf(buf, "0x%02lX", sub);
	_add_entry(dlg, "  Sub ID", buf);

	// Reserved
	res	= HPDI32_FRR_RESERVED_DECODE(frr);
	sprintf(buf, "0x%02lX", res);
	_add_entry(dlg, "  Reserved", buf);

	if ((did == HPDI32_DEVICE_ID_32) && (res & HPDI32_FRR_RESERVED_PMC))
	{
		sprintf(buf, "    0x%02lX", (long) HPDI32_FRR_RESERVED_PMC);
		res	&= ~HPDI32_FRR_RESERVED_PMC;
		_add_entry(dlg, buf, "Form Factor is PMC");
	}

	if ((did == HPDI32_DEVICE_ID_64) && (res & HPDI32_FRR_RESERVED_FW_64))
	{
		sprintf(buf, "    0x%02lX", (long) HPDI32_FRR_RESERVED_FW_64);
		res	&= ~HPDI32_FRR_RESERVED_FW_64;
		_add_entry(dlg, buf, "Firmware is 64-bit.");
	}

	if (res & HPDI32_FRR_FSR)
	{
		sprintf(buf, "    0x%02lX", (long) HPDI32_FRR_FSR);
		res	&= ~HPDI32_FRR_FSR;
		_add_entry(dlg, buf, "The Feature Set Register is present.");
	}

	if (res)
	{
		sprintf(buf, "    0x%02lX", (long) res);
		_add_entry(dlg, buf, "UNRECOGNIZED BITS");
	}
}



/*****************************************************************************
*
*	Function:	_frr_updated_id
*
*	Purpose:
*
*		Report the Firmware Revision Register identification.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		frr			The value from the Firmware Revision Register.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _frr_updated_id(HWND dlg, U32 frr)
{
	long	bs;
	char	buf[128];
	long	bv;
	long	con;
	long	ff;
	long	fr;
	long	fsr;
	long	hr;
	long	res;

	// Firmware Revision
	fr	= HPDI32_FRR_SPEC_FIRMWARE_DECODE(frr);
	sprintf(buf, "0x%02lX", fr);
	_add_entry(dlg, "  Firmware Revision", buf);

	// Board Version
	bv	= HPDI32_FRR_SPEC_BOARD_VER_DECODE(frr);
	sprintf(buf, "0x%02lX", bv);
	_add_entry(dlg, "  Board Version", buf);

	// Hardware Revision
	hr	= HPDI32_FRR_SPEC_HW_REV_DECODE(frr);
	sprintf(buf, "0x%02lX", hr);

	if (hr == 0)
		strcat(buf, " (no revisions)");
	else
		sprintf(buf + strlen(buf), " (Rev. %c)", (char) ('A' + hr - 1));

	_add_entry(dlg, "  Hardware Revision", buf);

	// Form Factor
	ff	= HPDI32_FRR_SPEC_FF_DECODE(frr);
	sprintf(buf, "0x%01lX", ff);

	if (ff == HPDI32_FRR_SPEC_FF_PCI)
		strcat(buf, " (PCI)");
	else if (ff == HPDI32_FRR_SPEC_FF_PMC)
		strcat(buf, " (PMC)");
	else if (ff == HPDI32_FRR_SPEC_FF_CPCI)
		strcat(buf, " (cPCI)");
	else if (ff == HPDI32_FRR_SPEC_FF_PC104P)
		strcat(buf, " (PC-104+)");
	else
		strcat(buf, " (NOT RECOGNIZED)");

	_add_entry(dlg, "  Form Factor", buf);

	// Bus Size
	bs	= HPDI32_FRR_SPEC_BUS_SIZE_DECODE(frr);
	sprintf(buf, "0x%01lX", bs);

	if (bs == HPDI32_FRR_SPEC_BUS_SIZE_64)
		strcat(buf, " (64-bit)");
	else
		strcat(buf, " (32-bit)");

	_add_entry(dlg, "  Bus Size", buf);

	// Reserved
	res	= HPDI32_FRR_SPEC_UNUSED_DECODE(frr);
	sprintf(buf, "0x%01lX", res);

	if (res)
		strcat(buf, "  THIS BIT SHOULD NOT BE SET.");

	_add_entry(dlg, "  Unused", buf);

	// Conformant
	con	= HPDI32_FRR_SPEC_CONFORM_DECODE(frr);
	sprintf(buf, "0x%01lX", con);

	if (con)
		strcat(buf, "Yes");
	else
		strcat(buf, "No");

	_add_entry(dlg, "  Conformant", buf);

	// Feature Set Register
	fsr	= HPDI32_FRR_SPEC_FSR_DECODE(frr);
	sprintf(buf, "0x%01lX", fsr);

	if (fsr)
		strcat(buf, "Yes");
	else
		strcat(buf, "No");

	_add_entry(dlg, "  Feature Set Register", buf);
}



/*****************************************************************************
*
*	Function:	_frr_id
*
*	Purpose:
*
*		Report the Firmware Revision Register identification.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		did			The PCI Device ID Register value.
*
*		frr			The Firmware Revision Register value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _frr_id(HWND dlg, _id_t* data, U32 did, U32* frr)
{
	char		buf[128];
	const char*	name	= "Firmware Rev. Register";
	U32			stat;
	U32			status	= GSC_SUCCESS;

	stat	= hpdi32_reg_read(data->device->handle, HPDI32_FRR, frr);
	status	= (status == GSC_SUCCESS) ? stat : status;
	sprintf(buf, "0x%08lX", (long) frr[0]);

	if (frr[0] & HPDI32_FRR_CONFORMANT)
	{
		strcat(buf, " (updated specification)");
		_add_entry(dlg, name, buf);
		_frr_updated_id(dlg, frr[0]);
	}
	else
	{
		strcat(buf, " (original format)");
		_add_entry(dlg, name, buf);
		_frr_original_id(dlg, did, frr[0]);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_fsr_id
*
*	Purpose:
*
*		Report the Feature Set Register information.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		frr			The value from the Firmware Revision Register.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _fsr_id(HWND dlg, _id_t* data, U32 frr)
{
	char	buf[128];
	U32		fsr;
	U32		status	= GSC_SUCCESS;

	if (frr & HPDI32_FRR_FSR)
	{
		// Feature Set Register
		status	= hpdi32_reg_read(data->device->handle, HPDI32_FSR, &fsr);
		sprintf(buf, "0x%08lX", fsr);
		_add_entry(dlg, "Feature Set Register", buf);

		// Tx FIFO Size Register and Rx FIFO Size Register
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_TFSR_RFSR);
		strcat(buf, (fsr & HPDI32_FSR_TFSR_RFSR) ? "YES" : "NO ");
		_add_entry(dlg, "  FIFO Size Registers", buf);
		fsr	&= ~HPDI32_FSR_TFSR_RFSR;

		// Tx FIFO Words Register and Rx FIFO Words Register
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_TFWR_RFWR);
		strcat(buf, (fsr & HPDI32_FSR_TFWR_RFWR) ? "YES" : "NO ");
		_add_entry(dlg, "  FIFO Words Registers", buf);
		fsr	&= ~HPDI32_FSR_TFWR_RFWR;

		// Interrupt Edge/Level Register and Interrupt Hi/Low Register.
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_IELR_IHLR);
		strcat(buf, (fsr & HPDI32_FSR_IELR_IHLR) ? "YES" : "NO ");
		_add_entry(dlg, "  IRQ Config Registers", buf);
		fsr	&= ~HPDI32_FSR_IELR_IHLR;

		// GPIO 0-5 support.
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_GPIO_0_5);
		strcat(buf, (fsr & HPDI32_FSR_GPIO_0_5) ? "YES" : "NO ");
		_add_entry(dlg, "  GPIO 0-5 Support", buf);
		fsr	&= ~HPDI32_FSR_GPIO_0_5;

		// Tx DMDMA on Channel 1
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_DMDMA_CH1);
		strcat(buf, (fsr & HPDI32_FSR_DMDMA_CH1) ? "YES" : "NO ");
		_add_entry(dlg, "  Tx DMDMA on Chan. 1", buf);
		fsr	&= ~HPDI32_FSR_DMDMA_CH1;

		// Tx Overrun, Rx Overrun and Rx Under Run
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_OVR_UNDR_RUN);
		strcat(buf, (fsr & HPDI32_FSR_OVR_UNDR_RUN) ? "YES" : "NO ");
		_add_entry(dlg, "  Tx/Rx Over/Under Run", buf);
		fsr	&= ~HPDI32_FSR_OVR_UNDR_RUN;

		// GPIO 6 support. This bit is GPIO 6 and Tx Auto Stop.
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_GPIO6_TXAS);
		strcat(buf, (fsr & HPDI32_FSR_GPIO6_TXAS) ? "YES" : "NO ");
		_add_entry(dlg, "  GPIO 6 Support", buf);

		// Tx Auto Stop Control
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_GPIO6_TXAS);
		strcat(buf, (fsr & HPDI32_FSR_GPIO6_TXAS) ? "YES" : "NO ");
		_add_entry(dlg, "  Tx Auto Stop Control", buf);
		fsr	&= ~HPDI32_FSR_GPIO6_TXAS;

		// Single Cycle Disable
		sprintf(buf, "0x%08lX ", (long) HPDI32_FSR_SCD);
		strcat(buf, (fsr & HPDI32_FSR_SCD) ? "YES" : "NO ");
		_add_entry(dlg, "  Single Cycle Disable", buf);
		fsr	&= ~HPDI32_FSR_SCD;

		if (fsr)
		{
			sprintf(buf, "0x%08lX", (long) fsr);
			_add_entry(dlg, "  UNRECOGNIZED BITS", buf);
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_identify_device
*
*	Purpose:
*
*		Fill the window with the identification information.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appripriate error status.
*
*****************************************************************************/

static U32 _identify_device(HWND dlg, _id_t* data)
{
	char		buf[1024];
	HWND		control;
	LRESULT		count;
	HDC			dc;
	U32			did;
	U32			frr;
	int			i;
	int			j;
	LRESULT		sel;
	U32			stat;
	U32			status	= GSC_SUCCESS;
	TEXTMETRIC	tm;
	LRESULT		top;
	int			width;

	control	= GetDlgItem(dlg, IDC_DEVICE_ID_LIST);
	SendMessage(control, WM_SETREDRAW, FALSE, 0);
	sel		= SendMessage(control, LB_GETCURSEL, 0, 0);
	top		= SendMessage(control, LB_GETTOPINDEX, 0, 0);
	SendMessage(control, LB_RESETCONTENT, 0, 0);

	stat	= _pci_id(dlg, data, &did);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _jumper_id(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _frr_id(dlg, data, did, &frr);
	status	= (status == GSC_SUCCESS) ? stat : status;

	if (frr & HPDI32_FRR_FSR)
	{
		stat	= _fsr_id(dlg, data, frr);
		status	= (status == GSC_SUCCESS) ? stat : status;
	}

	count	= SendMessage(control, LB_GETCOUNT, 0, 0);
	width	= 0;
	dc		= GetDC(control);
	GetTextMetrics(dc, &tm);
	ReleaseDC(control, dc);

	for (i = 0; i < count; i++)
	{
		SendMessage(control, LB_GETTEXT, (WPARAM) i, (LPARAM) buf);
		j	= (int) strlen(buf);

		if (width < j)
			width	= j;
	}

	width	= (width + 2) * tm.tmAveCharWidth;
	SendMessage(control, LB_SETHORIZONTALEXTENT, (WPARAM) width, 0);
	SendMessage(control, LB_SETCURSEL, (WPARAM) sel, 0);
	SendMessage(control, LB_SETTOPINDEX, (WPARAM) top, 0);
	SendMessage(control, WM_SETREDRAW, TRUE, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window's controls per the state of the device
*		in use, if any.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device structure to access.
*
*		msg		Any information message to appear in the status box.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	HWND	control;
	_id_t*	data;
	HMENU	menu;
	U32		status	= GSC_SUCCESS;

	data	= (_id_t*) GET_WINDOW_USER_DATA(dlg);

	data->status_local[0]	= 0;
	set_win_title(dlg, data->device, TRUE, TRUE, "Device ID");
	menu	= GetMenu(dlg);

	if (device->handle)
	{
		EnableMenuItem(menu, IDC_SAVE, MF_BYCOMMAND | MF_ENABLED);
		status	= _identify_device(dlg, data);
	}
	else
	{
		EnableMenuItem(menu, IDC_SAVE, MF_BYCOMMAND | MF_GRAYED);
		control		= GetDlgItem(dlg, IDC_DEVICE_ID_LIST);
		SendMessage(control, LB_RESETCONTENT, 0, 0);
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	if (status != GSC_SUCCESS)
		error_status(dlg, IDC_DEVICE_ID_STATUS, status);
	else if ((msg) && (msg[0]))
		SetDlgItemText(dlg, IDC_DEVICE_ID_STATUS, msg);
	else
		SetDlgItemText(dlg, IDC_DEVICE_ID_STATUS, data->status_local);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id					flags						*/
		{ -1,					COPY_EOL | COPY_UNDERLINE	},

		{ IDC_DEVICE_ID_LIST,	COPY_WIDTH | COPY_LIST_BOX	},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDCANCEL,				COPY_EOL					},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_DEVICE_ID_STATUS,	COPY_EOL					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the window.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	_id_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (_id_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	int		id	= LOWORD(wParam);
	LRESULT	ret	= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages's long size parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND dlg, LPARAM lParam)
{
	HWND		control;
	MINMAXINFO*	mmi		= (MINMAXINFO*) lParam;
	RECT		rect;

	// Height
	control	= GetDlgItem(dlg, IDC_DEVICE_ID_STATUS);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	= rect.bottom;

	control	= GetDlgItem(dlg, IDC_DEVICE_ID_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	-= rect.bottom;
	mmi->ptMinTrackSize.y	+= rect.top;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.y	-= rect.top;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	+= 3 * (rect.bottom - rect.top);

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	HWND	control;
	_id_t*	data;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (_id_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control		= GetDlgItem(dlg, IDC_DEVICE_ID_LIST);
	SendMessage(control, WM_SETFONT, (WPARAM) data->device->font.fixed, 0);

	win_pos_restore(dlg, SECTION_NAME, TRUE);
	_wm_size(dlg);
	_refresh_window(dlg, data->device, NULL);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND dlg)
{
	_id_t*	data	= (_id_t*) GET_WINDOW_USER_DATA(dlg);

	static	align_t	list[]	=
	{
		// The outter frame.
		{// 0
		/* id		*/	IDC_DEVICE_ID_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-1
		},
		{// 1
		/* id		*/	IDC_DEVICE_ID_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_T,
		/* value	*/	0
		},
		{// 2
		/* id		*/	IDC_DEVICE_ID_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R | ALIGN_B_B,
		/* value	*/	1
		},

		// The status text.
		{// 3
		/* id		*/	IDC_DEVICE_ID_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 4
		/* id		*/	IDC_DEVICE_ID_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 5
		/* id		*/	IDC_DEVICE_ID_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_B_B | ALIGN_H,
		/* value	*/	- MARGIN
		},

		// The status frame.
		{// 6
		/* id		*/	IDC_DEVICE_ID_FRAME_2,
		/* ref		*/	IDC_DEVICE_ID_FRAME_1,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R | ALIGN_B_B,
		/* value	*/	0
		},
		{// 7
		/* id		*/	IDC_DEVICE_ID_FRAME_2,
		/* ref		*/	IDC_DEVICE_ID_STATUS,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	- MARGIN
		},

		// The SAVE button.
		{// 8
		/* id		*/	IDCANCEL,
		/* ref		*/	0,
		/* flags	*/	ALIGN_HC_C | ALIGN_W,
		/* value	*/	0
		},
		{// 9
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_DEVICE_ID_FRAME_2,
		/* flags	*/	ALIGN_B_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The list box.
		{// 10
		/* id		*/	IDC_DEVICE_ID_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 11
		/* id		*/	IDC_DEVICE_ID_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_T,
		/* value	*/	-999	// see below
		},
		{// 12
		/* id		*/	IDC_DEVICE_ID_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 13
		/* id		*/	IDC_DEVICE_ID_LIST,
		/* ref		*/	IDCANCEL,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	-999	// see below
		},

		{ 0, 0, 0, 0 }
	};

	list[3].value	= data->device->margin;
	list[4].value	= - data->device->margin;
	list[9].value	= - data->device->margin;
	list[10].value	= data->device->margin;
	list[11].value	= 1 + data->device->margin;
	list[12].value	= - data->device->margin;
	list[13].value	= - data->device->margin;

	align_control_list(dlg, list);
	InvalidateRect(dlg, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_GETMINMAXINFO:

			ret	= _wm_getminmaxinfo(dlg, lParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SIZE:

			ret	= _wm_size(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HMENU	menu	= GetMenu(dlg);;

	if (device->handle)
		EnableMenuItem(menu, IDC_DEVICE_ID, MF_BYCOMMAND | MF_ENABLED);
	else
		EnableMenuItem(menu, IDC_DEVICE_ID, MF_BYCOMMAND | MF_GRAYED);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	_id_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device	= device;
		data.dlg	= CreateDialogParam(device->instance,
										(LPCTSTR) IDD_DEVICE_ID,
										dlg,
										(DLGPROC) _dlg_proc,
										(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	device_id_service
*
*	Purpose:
*
*		Implement the common service entry point for the Device ID services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT device_id_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


