// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/fifo.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for common FIFO configuration options.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "rx.h"
#include "copy.h"
#include "fifo.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	FIFO_SIZE_K		1024



/* variables	*************************************************************/

static const value_range_t	_almost_value_range	=
{
	/* min	*/	0,
	/* max	*/	HPDI32_FIFO_ALMOST_LEVEL_MAX,
	/* k	*/	1024
};

static const value_t	_almost_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_almost_value_range
	}
};

static const parm_entry_t	_almost_empty	=
{
	/* title			*/	"FIFO Almost Empty",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_FIFO_ALMOST_LEVEL),
	/* suffix			*/	"_AE",
	/* which			*/	HPDI32_WHICH_AE,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_almost_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls the FIFO fill level at which the Almost Empty"
	" status becomes active. The value entered refers to the number of"
	" samples at or below which the status becomes active. When the FIFO"
	" contains this number or fewer samples, then the status becomes active."
};

static const parm_entry_t	_almost_full	=
{
	/* title			*/	"FIFO Almost Full",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_FIFO_ALMOST_LEVEL),
	/* suffix			*/	"_AF",
	/* which			*/	HPDI32_WHICH_AF,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_almost_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls the FIFO fill level at which the Almost Full"
	" status becomes active. The value entered refers to the number of"
	" samples from full, at or above which the status becomes active. When"
	" the FIFO can receive this number or fewer additional samples before"
	" becoming full, then the status becomes active."
};

static	const parm_entry_t*	_fifo_parm_list[]	=
{
	&_almost_empty,
	&_almost_full
};

const parm_list_t	fifo_tx_parm_list	=
{
	/* list		*/	_fifo_parm_list,
	/* size		*/	ARRAY_ELEMENTS(_fifo_parm_list),
	/* suffix	*/	"_TX",
	/* which	*/	HPDI32_WHICH_TX
};

const parm_list_t	fifo_rx_parm_list	=
{
	/* list		*/	_fifo_parm_list,
	/* size		*/	ARRAY_ELEMENTS(_fifo_parm_list),
	/* suffix	*/	"_RX",
	/* which	*/	HPDI32_WHICH_RX
};



/*****************************************************************************
*
*	Function:	_get_state_info
*
*	Purpose:
*
*		Get the state info we need about the device.
*
*	Arguments:
*
*		device		The device data.
*
*		which		The "which" bit for the FIFO to access.
*
*		known		Tell the caller if the FIFO size is known.
*
*		fifo		Tell the caller the size of the FIFO.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _get_state_info(device_t* device, U32 which, int* known, U32* size)
{
	U32	feature;
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= HPDI32_FIFO_SIZE__GET(device->handle, which, size);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_MISC_FEATURES__FIFO_SIZE(device->handle, &feature);
	status	= (status == GSC_SUCCESS) ? stat : status;

	if (feature == HPDI32_MISC_FEATURES_ABSENT)
	{
		known[0]	= FALSE;
		size[0]		= 0;
	}
	else
	{
		known[0]	= TRUE;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_get_fifo_status_name
*
*	Purpose:
*
*		Translate a FIFO Status value to a descriptive text string.
*
*	Arguments:
*
*		status	The value to translate.
*
*	Returned:
*
*		!NULL		A singple text string.
*
*****************************************************************************/

static const char* _get_fifo_status_name(U32 status)
{
	const char*	psz;

	switch (status)
	{
		default:
		case HPDI32_FIFO_STATUS_EMPTY:			psz	= "Empty";			break;
		case HPDI32_FIFO_STATUS_ALMOST_EMPTY:	psz	= "Almost Empty";	break;
		case HPDI32_FIFO_STATUS_MEDIAN:			psz	= "Median";			break;
		case HPDI32_FIFO_STATUS_ALMOST_FULL:	psz	= "Almost Full";	break;
		case HPDI32_FIFO_STATUS_FULL:			psz	= "Full";			break;
	}

	return(psz);
}



/*****************************************************************************
*
*	Function:	_size_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _size_device_read(fifo_data_t* data)
{
	U32	status;

	status	= _get_state_info(	data->device,
								data->which,
								&data->size_known,
								&data->size.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_size_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _size_dialog_write(HWND dlg, fifo_data_t* data)
{
	char			k[32];
	unsigned int	flags;
	char			size[32];

	if (data->device->handle == NULL)
	{
		flags	= CTRL_DISABLE;
		size[0]	= 0;
		k[0]	= 0;
	}
	else if (data->size_known == FALSE)
	{
		flags	= CTRL_ENABLE;
		strcpy(size, "UNKNOWN");
		k[0]	= 0;
	}
	else if (data->size.setting == 0)
	{
		flags	= CTRL_ENABLE;
		strcpy(size, "NO FIFOS");
		k[0]	= 0;
	}
	else
	{
		flags	= CTRL_ENABLE;
		format_u32(data->size.setting, FIFO_SIZE_K, size);
		sprintf(k, "(1K = %d)", (int) FIFO_SIZE_K);
	}

	ctrl_menu_set(dlg, IDC_FIFO_SIZE_TITLE, flags);
	ctrl_menu_set(dlg, IDC_FIFO_SIZE_TEXT, flags);
	ctrl_menu_set(dlg, IDC_FIFO_SIZE_INFO, flags);
	ctrl_menu_set_text(dlg, IDC_FIFO_SIZE_TEXT, size);
	ctrl_menu_set_text(dlg, IDC_FIFO_SIZE_INFO, k);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_almost_empty_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _almost_empty_device_read(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_FIFO_ALMOST_LEVEL__GET(data->device->handle,
											data->which | HPDI32_WHICH_AE,
											&data->almost_empty.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_almost_empty_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _almost_empty_device_write(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_FIFO_ALMOST_LEVEL__SET(data->device->handle,
											data->which | HPDI32_WHICH_AE,
											data->almost_empty.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_almost_empty_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _almost_empty_dialog_read(HWND dlg, fifo_data_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_FIFO_ALMOST_EMPTY_EDIT,
								_almost_value_range.k,
								&data->almost_empty.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_almost_empty_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _almost_empty_dialog_write(HWND dlg, fifo_data_t* data)
{
	char			ae[32];
	unsigned int	flags;
	char			k[32];

	if (data->device->handle == NULL)
	{
		flags	= CTRL_DISABLE;
		ae[0]	= 0;
		k[0]	= 0;
	}
	else if ((data->size_known) && (data->size.setting == 0))
	{
		flags	= CTRL_DISABLE;
		ae[0]	= 0;
		k[0]	= 0;
	}
	else
	{
		flags	= CTRL_ENABLE;
		format_u32(data->almost_empty.setting, _almost_value_range.k, ae);
		sprintf(k, "(1K = %d)", (int) _almost_value_range.k);
	}

	ctrl_menu_set(dlg, IDC_FIFO_ALMOST_EMPTY_TITLE, flags);
	ctrl_menu_set(dlg, IDC_FIFO_ALMOST_EMPTY_EDIT, flags);
	ctrl_menu_set(dlg, IDC_FIFO_ALMOST_EMPTY_INFO, flags);
	ctrl_menu_set_text(dlg, IDC_FIFO_ALMOST_EMPTY_INFO, k);

	if (data->almost_empty.ignore == FALSE)
	{
		data->almost_empty.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_FIFO_ALMOST_EMPTY_EDIT, ae);
		data->almost_empty.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_almost_full_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _almost_full_device_read(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_FIFO_ALMOST_LEVEL__GET(data->device->handle,
											data->which | HPDI32_WHICH_AF,
											&data->almost_full.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_almost_full_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _almost_full_device_write(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_FIFO_ALMOST_LEVEL__SET(data->device->handle,
											data->which | HPDI32_WHICH_AF,
											data->almost_full.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_almost_full_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _almost_full_dialog_read(HWND dlg, fifo_data_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_FIFO_ALMOST_FULL_EDIT,
								_almost_value_range.k,
								&data->almost_full.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_almost_full_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _almost_full_dialog_write(HWND dlg, fifo_data_t* data)
{
	char			af[32];
	unsigned int	flags;
	char			k[32];

	if (data->device->handle == NULL)
	{
		flags	= CTRL_DISABLE;
		af[0]	= 0;
		k[0]	= 0;
	}
	else if ((data->size_known) && (data->size.setting == 0))
	{
		flags	= CTRL_DISABLE;
		af[0]	= 0;
		k[0]	= 0;
	}
	else
	{
		flags	= CTRL_ENABLE;
		format_u32(data->almost_full.setting, _almost_value_range.k, af);
		sprintf(k, "(1K = %d)", (int) _almost_value_range.k);
	}

	ctrl_menu_set(dlg, IDC_FIFO_ALMOST_FULL_TITLE, flags);
	ctrl_menu_set(dlg, IDC_FIFO_ALMOST_FULL_EDIT, flags);
	ctrl_menu_set(dlg, IDC_FIFO_ALMOST_FULL_INFO, flags);
	ctrl_menu_set_text(dlg, IDC_FIFO_ALMOST_FULL_INFO, k);

	if (data->almost_full.ignore == FALSE)
	{
		data->almost_full.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_FIFO_ALMOST_FULL_EDIT, af);
		data->almost_full.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_status_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _status_device_read(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_FIFO_STATUS__GET(	data->device->handle,
										data->which,
										&data->status.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_status_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _status_dialog_write(HWND dlg, fifo_data_t* data)
{
	unsigned int	flags;
	const char*		psz;

	if ((data->device->handle == NULL) ||
		((data->size_known) && (data->size.setting == 0)))
	{
		flags	= CTRL_DISABLE;
		psz		= "";
	}
	else
	{
		flags	= CTRL_ENABLE;
		psz		= _get_fifo_status_name(data->status.setting);
	}

	ctrl_menu_set(dlg, IDC_FIFO_STATUS_TITLE, flags);
	ctrl_menu_set(dlg, IDC_FIFO_STATUS_TEXT, flags);
	ctrl_menu_set_text(dlg, IDC_FIFO_STATUS_TEXT, psz);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_overrun_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _overrun_device_read(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_CONFIG_GET(data->device->handle,
								data->overrun.parm,
								0,
								&data->overrun.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_overrun_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _overrun_dialog_write(HWND dlg, fifo_data_t* data)
{
	if (data->device->handle)
	{
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_TITLE, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_STATUS, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_CLEAR, CTRL_ENABLE);

		if (data->overrun.setting == data->overrun.yes)
		{
			data->overrun_color	= data->device->paint.red.color;
			ctrl_menu_set_text(dlg, IDC_FIFO_OVERRUN_STATUS, "Yes");
		}
		else
		{
			data->overrun_color	= data->device->paint.black.color;
			ctrl_menu_set_text(dlg, IDC_FIFO_OVERRUN_STATUS, "No");
		}
	}
	else
	{
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_TITLE, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_STATUS, CTRL_DISABLE | CTRL_NULL);
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_CLEAR, CTRL_DISABLE);
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_under_run_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _under_run_device_read(fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_CONFIG_GET(data->device->handle,
								data->under_run.parm,
								0,
								&data->under_run.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_under_run_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _under_run_dialog_write(HWND dlg, fifo_data_t* data)
{
	if (data->device->handle)
	{
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_TITLE, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_STATUS, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_CLEAR, CTRL_ENABLE);

		if (data->under_run.setting == data->under_run.yes)
		{
			data->under_run_color	= data->device->paint.red.color;
			ctrl_menu_set_text(dlg, IDC_FIFO_UNDER_RUN_STATUS, "Yes");
		}
		else
		{
			data->under_run_color	= data->device->paint.black.color;
			ctrl_menu_set_text(dlg, IDC_FIFO_UNDER_RUN_STATUS, "No");
		}
	}
	else
	{
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_TITLE, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_STATUS, CTRL_DISABLE | CTRL_NULL);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_CLEAR, CTRL_DISABLE);
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(fifo_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _size_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _almost_empty_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _almost_full_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _status_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _overrun_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _under_run_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(fifo_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _almost_empty_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _almost_full_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, fifo_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _almost_empty_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _almost_full_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, fifo_data_t* data)
{
	unsigned int	flags;
	U32				stat;
	U32				status	= GSC_SUCCESS;

	stat	= _size_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _almost_empty_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _almost_full_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _status_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _overrun_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _under_run_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	if ((data->device->handle == NULL) ||
		((data->size_known) && (data->size.setting == 0)))
	{
		flags	= CTRL_DISABLE;
	}
	else
	{
		flags	= CTRL_ENABLE;
	}

	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	ctrl_menu_set(dlg, IDC_REFRESH, flags);
	ctrl_menu_set(dlg, IDC_RESET, flags);

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	fifo_data_t*	data;
	U32				stat;
	U32				status	= GSC_SUCCESS;
	char			title[64];

	data	= (fifo_data_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(title, data->title);
	strcat(title, " FIFO");
	strcat(title, data->changes ? " *" : "");
	set_win_title(dlg, data->device, FALSE, TRUE, title);

	if (data->changes)
		ctrl_menu_set_text(dlg, IDCANCEL, "&Cancel");
	else
		ctrl_menu_set_text(dlg, IDCANCEL, "&Close");

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			// Update everything from the device.
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		data->was_open	= TRUE;
	}
	else
	{
		data->changes	= FALSE;
		data->was_open	= FALSE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, fifo_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id							flags										*/
		{ -1,							COPY_EOL | COPY_UNDERLINE					},

		{ IDC_FIFO_SIZE_TITLE,			COPY_WIDTH | 14								},
		{ IDC_FIFO_SIZE_TEXT,			0											},
		{ IDC_FIFO_SIZE_INFO,			COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ IDC_FIFO_ALMOST_EMPTY_TITLE,	COPY_WIDTH | 14								},
		{ IDC_FIFO_ALMOST_EMPTY_EDIT,	0											},
		{ IDC_FIFO_ALMOST_EMPTY_INFO,	COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ IDC_FIFO_ALMOST_FULL_TITLE,	COPY_WIDTH | 14								},
		{ IDC_FIFO_ALMOST_FULL_EDIT,	0											},
		{ IDC_FIFO_ALMOST_FULL_INFO,	COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ IDC_FIFO_STATUS_TITLE,		COPY_WIDTH | 14								},
		{ IDC_FIFO_STATUS_TEXT,			COPY_EOL | COPY_DISABLED					},

		{ IDC_FIFO_OVERRUN_TITLE,		COPY_WIDTH | 14								},
		{ IDC_FIFO_OVERRUN_STATUS,		0											},
		{ IDC_FIFO_OVERRUN_CLEAR,		COPY_INDENT | COPY_EOL | COPY_DISABLED		},

		{ IDC_FIFO_UNDER_RUN_TITLE,		COPY_WIDTH | 14 | COPY_IF_EXISTS			},
		{ IDC_FIFO_UNDER_RUN_STATUS,	COPY_IF_EXISTS								},
		{ IDC_FIFO_UNDER_RUN_CLEAR,		COPY_INDENT | COPY_EOL | COPY_DISABLED | COPY_IF_EXISTS	},

		{ 0,							COPY_UNDERLINE								},
		{ 0,							COPY_EOL									},

		{ IDOK,							COPY_EOL | COPY_WIDTH | 8 | COPY_DISABLED	},
		{ IDCANCEL,						COPY_EOL | COPY_WIDTH | 8 | COPY_DISABLED	},
		{ IDC_APPLY,					COPY_EOL | COPY_WIDTH | 8 | COPY_DISABLED	},
		{ IDC_DEFAULT,					COPY_EOL | COPY_WIDTH | 8 | COPY_DISABLED	},
		{ IDC_REFRESH,					COPY_EOL | COPY_WIDTH | 8 | COPY_DISABLED	},
		{ IDC_RESET,					COPY_EOL | COPY_WIDTH | 8 | COPY_DISABLED	}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Process clicking of the DEFAULT button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, fifo_data_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->almost_empty.setting	= data->almost_empty.defalt;
	data->almost_full.setting	= data->almost_full.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_reset
*
*	Purpose:
*
*		Process clicking of the RESET button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_reset(HWND dlg, fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_FIFO_RESET__YES(data->device->handle, data->which);
	data->changes	= FALSE;
	data->was_open	= FALSE;
	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	fifo_data_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (fifo_data_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		data->dlg	= NULL;
		win_pos_save(dlg, data->section_name);
	}

	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, fifo_data_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_idc_fifo_almost_empty_edit
*
*	Purpose:
*
*		Respond to activity in the Almost Empty edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_fifo_almost_empty_edit(HWND dlg, WPARAM wParam, fifo_data_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->almost_empty.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idc_fifo_almost_full_edit
*
*	Purpose:
*
*		Respond to activity in the Almost Full edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_fifo_almost_full_edit(HWND dlg, WPARAM wParam, fifo_data_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->almost_full.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idc_fifo_overrun_clear
*
*	Purpose:
*
*		Respond to a request to a clear an Overrun condition.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_fifo_overrun_clear(HWND dlg, fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_CONFIG_SET(data->device->handle,
								data->overrun.parm,
								0,
								data->overrun.clear);
	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_fifo_under_run_clear
*
*	Purpose:
*
*		Respond to a request to a clear an Under Run condition.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_fifo_under_run_clear(HWND dlg, fifo_data_t* data)
{
	U32	status;

	status	= HPDI32_CONFIG_SET(data->device->handle,
								data->under_run.parm,
								0,
								data->under_run.clear);
	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	fifo_data_t*	data	= (fifo_data_t*) GET_WINDOW_USER_DATA(dlg);
	int				id		= LOWORD(wParam);
	LRESULT			ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDC_FIFO_ALMOST_EMPTY_EDIT:

			_idc_fifo_almost_empty_edit(dlg, wParam, data);
			break;

		case IDC_FIFO_ALMOST_FULL_EDIT:

			_idc_fifo_almost_full_edit(dlg, wParam, data);
			break;

		case IDC_FIFO_OVERRUN_CLEAR:

			_idc_fifo_overrun_clear(dlg, data);
			break;

		case IDC_FIFO_UNDER_RUN_CLEAR:

			_idc_fifo_under_run_clear(dlg, data);
			break;

		case IDC_REFRESH:

			_refresh_window(dlg, data->device, NULL);
			break;

		case IDC_RESET:

			_idc_reset(dlg, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_ctlcolorstatic
*
*	Purpose:
*
*		Process a WM_CTLCOLORSTATIC message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		The brush handle to use for the background.
*
*****************************************************************************/

static LRESULT _wm_ctlcolorstatic(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	fifo_data_t*	data	= (fifo_data_t*) GET_WINDOW_USER_DATA(dlg);
	HDC				dc		= (HDC) wParam;
	HWND			cntrl;
	HWND			control	= (HWND) lParam;
	HGDIOBJ			obj		= NULL;

	cntrl	= GetDlgItem(dlg, IDC_FIFO_OVERRUN_STATUS);

	if (cntrl == control)
	{
		obj		= data->device->paint.btnface.brush;
		SetTextColor(dc, data->overrun_color);
		SetBkColor(dc, data->device->paint.btnface.color);
	}
	else
	{
		cntrl	= GetDlgItem(dlg, IDC_FIFO_UNDER_RUN_STATUS);

		if (cntrl == control)
		{
			obj		= data->device->paint.btnface.brush;
			SetTextColor(dc, data->under_run_color);
			SetBkColor(dc, data->device->paint.btnface.color);
		}
	}

	return((LRESULT) obj);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char			buf[32];
	HWND			control;
	fifo_data_t*	data;
	U32				status;
	U32				u32;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (fifo_data_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_FIFO_ALMOST_EMPTY_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	control	= GetDlgItem(dlg, IDC_FIFO_ALMOST_FULL_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "(1K = %d)", (int) _almost_value_range.k);
	ctrl_menu_set_text(dlg, IDC_FIFO_ALMOST_FULL_INFO, buf);

	status	= HPDI32_MISC_FEATURES__OVR_UNDR_RUN(data->device->handle, &u32);

	if (u32 == HPDI32_MISC_FEATURES_ABSENT)
	{
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_TITLE, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_STATUS, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_OVERRUN_CLEAR, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_TITLE, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_STATUS, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_CLEAR, CTRL_DELETE);
	}
	else if (data->under_run.parm == 0)
	{
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_TITLE, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_STATUS, CTRL_DELETE);
		ctrl_menu_set(dlg, IDC_FIFO_UNDER_RUN_CLEAR, CTRL_DELETE);
	}

	win_pos_restore(dlg, data->section_name, FALSE);
	main_refresh_add(dlg, _refresh_window);
	error_status(dlg, 0, status);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_CTLCOLORSTATIC:

			ret	= _wm_ctlcolorstatic(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND			control;
	fifo_data_t*	data	= (fifo_data_t*) wParam;
	char*			psz;
	U32				status;
	U32				width;

	data->device	= device;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, data->ctrl_id, CTRL_DISABLE);
			ctrl_menu_set(dlg, data->arrow_id, CTRL_DISABLE | CTRL_NULL);
			break;
		}

		// The device is open.
		status	= HPDI32_IO_DATA_SIZE__GET(device->handle, data->which, &width);
		error_status(dlg, IDC_INFO_TEXT, status);

		switch (width)
		{
			default:
			case HPDI32_IO_DATA_SIZE_8_BITS:

				psz	= "8-bits";
				break;

			case HPDI32_IO_DATA_SIZE_16_BITS:

				psz	= "16-bits";
				break;

			case HPDI32_IO_DATA_SIZE_32_BITS:

				psz	= "32-bits";
				break;
		}

		ctrl_menu_set(dlg, data->ctrl_id, CTRL_ENABLE);
		ctrl_menu_set(dlg, data->arrow_id, CTRL_ENABLE);
		ctrl_menu_set_text(dlg, data->arrow_id, psz);
		break;
	}

	control	= GetDlgItem(dlg, data->ctrl_id);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	fifo_data_t*	data	= (fifo_data_t*) wParam;

	if (data->dlg)	/*lint !e727 */
	{
		SetFocus(data->dlg);
	}
	else
	{
		data->changes				= FALSE;
		data->device				= device;
		data->almost_empty.ignore	= TRUE;
		data->almost_full.ignore	= TRUE;
		data->was_open				= FALSE;
		data->dlg	= CreateDialogParam(device->instance,
										(LPCTSTR) IDD_FIFO,
										dlg,
										(DLGPROC) _dlg_proc,
										(LPARAM) data);
		ShowWindow(data->dlg, SW_SHOWNORMAL);
		data->almost_empty.ignore	= FALSE;
		data->almost_full.ignore	= FALSE;
		_refresh_window(data->dlg, device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	fifo_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx FIFO services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT fifo_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	fifo_gui_paint
*
*	Purpose:
*
*		Update the appearance of a FIFO button.
*
*	Arguments:
*
*		control		The hande to the control
*
*		data		The control specific data we track.
*
*		fifo		The FIFO data structure for the FIFO being accessed.
*
*		overrun		Has there been a FIFO overrun?
*
*		underrun	Has there been a FIFO under run?
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void fifo_gui_paint(
	HWND			control,
	button_t*		data,
	fifo_data_t*	fifo,
	U32				overrun,
	U32				underrun)
{
	HBRUSH			brush;
	char			buf[128];
	COLORREF		color;
	HWND			dlg		=	GetParent(control);
	POINT			list[5];
	HPEN			pen;
	int				known;
	U32				level;
	U32				size;
	U32				status;
	int				w;
	int				wae;
	int				waf;
	int				we;
	int				wf;
	int				wm;
	int				x;
	int				y;

	y	= data->inside2.rect.top
		+ data->offset
		+ data->tmAscent;

	if (data->device->handle)
	{
		status	= _get_state_info(data->device, fifo->which, &known, &size);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (known)
		{
			strcpy(buf, "(");
			format_u32(size, FIFO_SIZE_K, buf + 1);
			strcat(buf, ")");
			w	= button_text_paint(control, data, -9999, -9999, buf);
			x	= ((data->inside2.rect.left + data->inside2.rect.right) / 2)
				- (w / 2)
				+ data->offset;
			button_text_paint(control, data, x, y, buf);
		}
	}

	y	+= data->tmAscent;

	if (data->device->handle)
	{
		if ((overrun) && (underrun))
			strcpy(buf, "Overrun! Under Run!");
		else if (overrun)
			strcpy(buf, "Overrun!");
		else if (underrun)
			strcpy(buf, "Under Run!");
		else
			buf[0]	= 0;

		if (buf[0])
		{
			color	= data->text2.color_fg;
			data->text2.color_fg	= data->device->paint.red.color;
			w	= button_text_paint(control, data, -9999, -9999, buf);
			x	= ((data->inside2.rect.left + data->inside2.rect.right) / 2)
				- (w / 2)
				+ data->offset;
			button_text_paint(control, data, x, y, buf);
			data->text2.color_fg	= color;
		}
	}

	pen	= SelectObject(data->dc, data->arrow2.pen);
	y		+= data->tmAscent + 2;
	list[0].x	= data->inside2.rect.left + 2 + data->offset;
	list[0].y	= y + data->offset;
	list[1].x	= data->inside2.rect.left + 2 + data->offset;
	list[1].y	= data->inside2.rect.bottom - 3 + data->offset;
	list[2].x	= data->inside2.rect.right - 3 + data->offset;
	list[2].y	= data->inside2.rect.bottom - 3 + data->offset;
	list[3].x	= data->inside2.rect.right - 3 + data->offset;
	list[3].y	= y + data->offset;
	list[4].x	= data->inside2.rect.left + 2 + data->offset;
	list[4].y	= y + data->offset;

	y		= list[1].y - data->tmAscent;

	x	= list[0].x;
	we	= button_text_paint(control, data, x, y, " E ");
	x	+= we;
	MoveToEx(data->dc, x, y, NULL);
	LineTo(data->dc, x, list[1].y);

	wae	= button_text_paint(control, data, x + 1, y, " AE ");
	x	+= wae;
	MoveToEx(data->dc, x, y, NULL);
	LineTo(data->dc, x, list[1].y);

	w	= button_text_paint(control, data, -9999, -9999, " AF  F ");
	x	= list[2].x - w;

	MoveToEx(data->dc, x - 1, y, NULL);
	LineTo(data->dc, x - 1, list[1].y);
	waf	= button_text_paint(control, data, x, y, " AF ");

	x	+= waf;
	MoveToEx(data->dc, x, y, NULL);
	LineTo(data->dc, x, list[1].y);
	wf	= button_text_paint(control, data, x, y, " F ");

	wm	= button_text_paint(control, data, -9999, -9999, "M");
	x	= ((list[0].x + list[2].x) / 2) - (wm / 2);
	button_text_paint(control, data, x, y, "M");

	Polyline(data->dc, list, ARRAY_ELEMENTS(list));

	if (data->device->handle)
	{
		status	= HPDI32_FIFO_STATUS__GET(data->device->handle, fifo->which, &level);
		error_status(dlg, IDC_INFO_TEXT, status);

		switch (level)
		{
			default:
			case HPDI32_FIFO_STATUS_EMPTY:

				x	= list[0].x + (we / 2);
				break;

			case HPDI32_FIFO_STATUS_ALMOST_EMPTY:

				x	= list[0].x + we + (wae / 2);
				break;

			case HPDI32_FIFO_STATUS_MEDIAN:

				x	= (list[0].x + list[2].x) / 2;
				break;

			case HPDI32_FIFO_STATUS_ALMOST_FULL:

				x	= list[2].x - wf - (waf / 2);
				break;

			case HPDI32_FIFO_STATUS_FULL:

				x	= list[2].x - (wf / 2);
				break;
		}

		list[0].x	= x;
		list[0].y	= list[0].y + 3;
		list[1].x	= x + (we / 4);
		list[1].y	= list[0].y;
		list[2].x	= x;
		list[2].y	= y - 2;
		list[3].x	= x - (we / 4);
		list[3].y	= list[0].y;
		list[4].x	= x;
		list[4].y	= list[0].y;

		brush	= SelectObject(data->dc, data->arrow2.brush);
		Polygon(data->dc, list, 5);
		SelectObject(data->dc, brush);
	}

	SelectObject(data->dc, pen);
}


