// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gpio.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for common GPIO services.
*
*****************************************************************************/

#include <windows.h>

#include "gpio.h"
#include "align.h"
#include "arrow.h"
#include "cable.h"
#include "resource.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_GPIO,
		/* ref		*/	IDC_GPIO_WRITE,
		/* flags	*/	ALIGN_L_L,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_GPIO,
		/* ref		*/	IDC_GPIO_3,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B | ALIGN_R_L,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	HWND	control;

	control	= GetDlgItem(dlg, IDC_GPIO);
	SendMessage(control, WM_SETFONT, (WPARAM) device->font.defalt_bl, 0);
	align_control_list(dlg, list);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_GPIO, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_GPIO, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh_x
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service for
*		the GPIO X window control.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh_x(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND			control;
	UINT			flags;
	const gpio_t*	gpio	= (const gpio_t*) wParam;
	U32				mode;
	U32				status;

	if (device->handle)
	{
		ctrl_menu_set(dlg, gpio->id_static, CTRL_ENABLE);
		ctrl_menu_set(dlg, gpio->id_arrow, CTRL_ENABLE);
		status	= HPDI32_CABLE_COMMAND_MODE__GET(device->handle, gpio->which, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);

		switch (mode)
		{
			default:
			case HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL:

				flags	= CTRL_DISABLE;
				static_arrow_direction_set(dlg, gpio->id_static, 0);
				arrow_type_set(dlg, gpio->id_arrow, ARROW_HORZ);
				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_IN:

				flags	= CTRL_ENABLE;
				static_arrow_direction_set(dlg, gpio->id_static, -1);
				arrow_type_set(dlg, gpio->id_arrow, ARROW_LEFT);
				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW:
			case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI:

				flags	= CTRL_ENABLE;
				static_arrow_direction_set(dlg, gpio->id_static, +1);
				arrow_type_set(dlg, gpio->id_arrow, ARROW_RIGHT);
				break;
		}

		ctrl_menu_set(dlg, gpio->id_arrow, flags);
	}
	else
	{
		ctrl_menu_set(dlg, gpio->id_static, CTRL_DISABLE);
		ctrl_menu_set(dlg, gpio->id_arrow, CTRL_DISABLE | CTRL_NULL);
		arrow_type_set(dlg, gpio->id_arrow, ARROW_HORZ);
	}

	control	= GetDlgItem(dlg, gpio->id_static);
	InvalidateRect(control, NULL, TRUE);
	control	= GetDlgItem(dlg, gpio->id_arrow);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_ctlcolorstatic
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_CTLCOLORSTATIC
*		service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		Use the default brush.
*		else	Use this brush handle for the background.
*
*****************************************************************************/

static LRESULT _service_wm_ctlcolorstatic(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HDC		dc	= (HDC) wParam;
	LRESULT	ret	= (LRESULT) device->paint.btnhighlt.brush;

	SetBkColor(dc, device->paint.btnhighlt.color);
	return(ret);
}



/*****************************************************************************
*
*	Function:	gpio_service
*
*	Purpose:
*
*		Implement the common service entry point for the GPIO services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		arg1	A service or servicer dependent value.
*
*		arg2	A service or servicer dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gpio_service(
	HWND					dlg,
	device_t*				device,
	service_type_t			service,
	WPARAM					arg1,
	LPARAM					arg2)
{
	static const service_call_t	list[]	=
	{
		/* service						function					*/
		{ SERVICE_INIT,					_service_init				},
		{ SERVICE_REFRESH,				_service_refresh			},
		{ SERVICE_WM_CTLCOLORSTATIC,	_service_wm_ctlcolorstatic	},
		{ SERVICE_END_LIST,				NULL						}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, arg1, arg2);
	return(ret);
}


/*****************************************************************************
*
*	Function:	gpio_x_service
*
*	Purpose:
*
*		Implement the common service entry point for the GPIO X services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gpio_x_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_REFRESH,	_service_refresh_x	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	gpio_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for GPIO GUI controls.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*		gpio		The data specific to the GPIO line being accessed.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gpio_gui_paint(HWND control, static_t* data, const gpio_t* gpio)
{
	static const wave_t	down[]	=
	{
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	up[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	HWND	dlg		= GetParent(control);
	U32		mode;
	U32		status;
	U8		value;
	int		x;
	int		y;

	x	= data->inside.rect.left
		+ data->arrow.offset
		+ data->arrow.width
		+ MARGIN;
	y	= data->inside.rect.top;

	if (data->device->handle)
	{
		status	= HPDI32_CABLE_COMMAND_MODE__GET(data->device->handle, gpio->which, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= hpdi32_gpio_read(data->device->handle, &value);
		error_status(dlg, IDC_INFO_TEXT, status);

		switch (mode)
		{
			default:
			case HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL:

				x	+= static_text_paint(control, data, x, y, " Signal is ");
				static_text_paint(control, data, x, y, gpio->fc_name);
				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_IN:

				if (value & gpio->mask)
				{
					x	+= static_text_paint(control, data, x, y, " Input High ");
					static_wave_paint(data, x, y, up);
				}
				else
				{
					x	+= static_text_paint(control, data, x, y, " Input Low ");
					static_wave_paint(data, x, y, down);
				}

				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW:

				x	+= static_text_paint(control, data, x, y, " Output Low ");
				static_wave_paint(data, x, y, down);
				break;

			case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI:

				x	+= static_text_paint(control, data, x, y, " Output High ");
				static_wave_paint(data, x, y, up);
				break;
		}
	}
}


