// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_disable.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the receiver's Rx Disable configuration option.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "arrow.h"
#include "resource.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_get_state_info
*
*	Purpose:
*
*		Get the state info we need about the device.
*
*	Arguments:
*
*		device		The device data.
*
*		tx_enabled	Tell the caller if the transmitter is enable.
*
*		rx_enabled	Tell the caller if the receiver is enable.
*
*		rx_fifo		Tell the caller if the receiver has FIFOs.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _get_state_info(
	device_t*	device,
	int*		tx_enabled,
	int*		rx_enabled,
	int*		rx_fifo)
{
	U32	enable;
	U32	feature;
	U32	size;
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat			= HPDI32_TX_ENABLE__GET(device->handle, &enable);
	status			= (status == GSC_SUCCESS) ? stat : status;
	tx_enabled[0]	= (enable == HPDI32_TX_ENABLE_YES) ? TRUE : FALSE;
	stat			= HPDI32_RX_ENABLE__GET(device->handle, &enable);
	status			= (status == GSC_SUCCESS) ? stat : status;
	rx_enabled[0]	= (enable == HPDI32_RX_ENABLE_YES) ? TRUE : FALSE;

	stat	= HPDI32_FIFO_SIZE__RX_GET(device->handle, &size);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_MISC_FEATURES__FIFO_SIZE(device->handle, &feature);
	status	= (status == GSC_SUCCESS) ? stat : status;

	if ((feature == HPDI32_MISC_FEATURES_ABSENT) || (size))
		rx_fifo[0]	= TRUE;
	else
		rx_fifo[0]	= FALSE;

	return(status);
}



/*****************************************************************************
*
*	Function:	_show_disable_button
*
*	Purpose:
*
*		Make sure the button is the DISABLE button.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _show_disable_button(HWND dlg)
{
	HWND	control;

	control	= GetDlgItem(dlg, IDC_RX_ENABLE);
	SetWindowLong(control, GWL_ID, IDC_RX_DISABLE);
	control	= GetDlgItem(dlg, IDC_RX_DISABLE);
	SetWindowText(control, "Disable");
}



/*****************************************************************************
*
*	Function:	_refresh_common
*
*	Purpose:
*
*		Update the state of the Rx Enable control for the GUI window.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device data.
*
*		tx_enabled	Tell the caller if the transmitter is enable.
*
*		rx_enabled	Tell the caller if the receiver is enable.
*
*		rx_fifo		Tell the caller if the receiver has FIFOs.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _refresh_common(
	HWND		dlg,
	device_t*	device,
	int*		tx_enabled,
	int*		rx_enabled,
	int*		rx_fifo)
{
	U32	status;

	if (device->handle)
		status	= _get_state_info(device, tx_enabled, rx_enabled, rx_fifo);
	else
		status	= GSC_SUCCESS;

	if (device->handle == NULL)
	{
		// No device opened.
		ctrl_menu_set(dlg, IDC_RX_DISABLE, CTRL_DISABLE);
	}
	else if (rx_fifo[0] == FALSE)
	{
		// No receive FIFOs.
		ctrl_menu_set(dlg, IDC_RX_DISABLE, CTRL_DISABLE);
	}
	else if (rx_enabled[0])
	{
		// The receiver is enabled.
		_show_disable_button(dlg);
		ctrl_menu_set(dlg, IDC_RX_DISABLE, CTRL_ENABLE);
	}
	else
	{
		// The receiver is disabled.
		ctrl_menu_set(dlg, IDC_RX_DISABLE, CTRL_DISABLE);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	int	fifo;
	int	rx;
	U32	status;
	int	tx;

	status	= _refresh_common(dlg, device, &tx, &rx, &fifo);
	error_status(dlg, 0, status);

	if (!device->handle)
		ctrl_menu_set(dlg, IDC_RX_ENABLE_ARROW, CTRL_DISABLE | CTRL_NULL);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32	status;

	status	= HPDI32_RX_ENABLE__NO(device->handle);
	error_status(dlg, 0, status);
	main_refresh(device, NULL);
	return(0);
}



/*****************************************************************************
*
*	Function:	rx_disable_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx Disable services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_disable_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


