// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/static.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for custom static recessed controls.
*
*****************************************************************************/

#include <windows.h>

#include "static.h"
#include "gui.h"



/*****************************************************************************
*
*	Function:	_arrow_paint
*
*	Purpose:
*
*		Paint an arrowhead per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The polyline data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _arrow_paint(HDC dc, const static_arrow_t* data)
{
	HBRUSH	brush;
	HPEN	pen;

	if (data->enable)
	{
		brush	= SelectObject(dc, data->brush);
		pen		= SelectObject(dc, data->pen);

		Polygon(dc, data->list, data->count);

		SelectObject(dc, brush);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_polyline_paint
*
*	Purpose:
*
*		Paint a polyline per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The polyline data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _polyline_paint(HDC dc, const static_polyline_t* data)
{
	HPEN	pen;

	if ((data->count) && (data->pen))
	{
		pen	= SelectObject(dc, data->pen);

		Polyline(dc, data->list, data->count);

		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_rect_paint
*
*	Purpose:
*
*		Paint a rectangle per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The rectangle data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _rect_paint(HDC dc, const static_rect_t* data)
{
	HBRUSH	brush;
	HPEN	pen;

	if (data->pen)
	{
		brush	= SelectObject(dc, data->brush);
		pen		= SelectObject(dc, data->pen);

		Rectangle(	dc,
					data->rect.left,
					data->rect.top,
					data->rect.right,
					data->rect.bottom);

		SelectObject(dc, brush);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_text_align
*
*	Purpose:
*
*		Compute the text alignment for one line of text.
*
*	Arguments:
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _text_align(static_t* data)
{
	HFONT		font;
	RECT		rect;
	int			width;

	font	= SelectObject(data->dc, data->text1.font);
	memset(&rect, 0, sizeof(rect));
	DrawText(data->dc, data->text1.text, -1, &rect, DT_CALCRECT);
	width	= RECT_WIDTH(rect);
	SelectObject(data->dc, font);

	if (data->style & SS_CENTER)
	{
		data->text2.x	= (RECT_WIDTH(data->client) - width) / 2;
		data->text1.x	= data->text2.x + 1;
	}
	else if (data->style & SS_RIGHT)
	{
		data->text2.x	= data->inside.rect.right - width;
		data->text1.x	= data->text2.x - 1;
	}
	else
	{
		data->text2.x	= data->inside.rect.left + 1;
		data->text1.x	= data->text2.x + 1;
	}

	data->text2.y		= data->inside.rect.top;
	data->text1.y		= data->text2.y + 1;
}



/*****************************************************************************
*
*	Function:	_text_paint
*
*	Purpose:
*
*		Paint text per the given structure.
*
*	Arguments:
*
*		dc		The device context to use.
*
*		data	The text data to use.
*
*	Returned:
*
*		The width of the painted text in pixels.
*
*****************************************************************************/

static int _text_paint(HDC dc, const static_text_t* data)
{
	UINT		align;
	COLORREF	color_bg;
	COLORREF	color_fg;
	HFONT		font;
	int			mode;
	RECT		rect;
	int			width	= 0;

	if ((data->enable) && (data->length))
	{
		align		= SetTextAlign(dc, TA_LEFT | TA_TOP);
		color_bg	= SetBkColor(dc, data->color_bg);
		color_fg	= SetTextColor(dc, data->color_fg);
		font		= SelectObject(dc, data->font);
		mode		= SetBkMode(dc, TRANSPARENT);

		ExtTextOut(	dc,
					data->x,
					data->y,
					ETO_CLIPPED,
					&data->rect,
					data->text,
					data->length,
					NULL);

		DrawText(dc, data->text, -1, &rect, DT_CALCRECT);
		width	= RECT_WIDTH(rect);

		SetTextAlign(dc, align);
		SetBkColor(dc, color_bg);
		SetTextColor(dc, color_fg);
		SelectObject(dc, font);
		SetBkMode(dc, mode);
	}

	return(width);
}



/*****************************************************************************
*
*	Function:	_wave_paint
*
*	Purpose:
*
*		Paint a portion of a waveform per the given data.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*		x		The starting X coordinate.
*
*		y		The starting Y coordinate.
*
*		wave	The info for the waveform portion we're to process.
*
*		dx		The unit width in pixels.
*
*		dy		The waveform height in pixels.
*
*	Returned:
*
*		The ending X coordinate.
*
*****************************************************************************/

static int _wave_paint(
	static_t*		data,
	int				x,
	int				y,
	const wave_t*	wave,
	int				dx,
	int				dy)
{
	int		count	= wave->count;
	int		y2		= y + dy;

	x	+= wave->skip * dx;
	MoveToEx(data->dc, x, y, NULL);

	for (; count--;)
	{
		if (wave->w1)
		{
			x	+= dx * wave->w1;
			LineTo(data->dc, x, y);
		}

		if (wave->w2)
		{
			LineTo(data->dc, x, y2);
			x	+= dx * wave->w2;
			LineTo(data->dc, x, y2);
			LineTo(data->dc, x, y);
		}

		if (count == 0)
			LineTo(data->dc, x, y);
	}

	return(x);
}



/*****************************************************************************
*
*	Function:	_control_paint
*
*	Purpose:
*
*		Paint the given button.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _control_paint(HWND control, static_t* data)
{
	_polyline_paint(data->dc, &data->border.outter);
	_polyline_paint(data->dc, &data->border.ul_out);
	_polyline_paint(data->dc, &data->border.lr_out);
	_polyline_paint(data->dc, &data->border.ridge);
	_polyline_paint(data->dc, &data->border.ul_in);
	_rect_paint(data->dc, &data->inside);
	_text_paint(data->dc, &data->text1);
	_text_paint(data->dc, &data->text2);
	_arrow_paint(data->dc, &data->arrow);

	if (data->paint)
		(data->paint)(control, data);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		control		The control handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND control)
{
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);

	if (data)
	{
		SET_WINDOW_USER_DATA(control, 0);
		free(data);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_enable
*
*	Purpose:
*
*		Process WM_ENABLE messages.
*
*	Arguments:
*
*		control		The control handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_enable(HWND control, WPARAM wParam)
{
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);
	int			enabled	= wParam ? TRUE : FALSE;

	if (data == NULL)
	{
	}
	else if (enabled)
	{
		data->arrow.brush		= data->device->paint.btntext.brush;
		data->arrow.pen			= data->device->paint.btntext.pen;

		data->text1.color_fg	= data->device->paint.btnhighlt.color;
		data->text1.enable		= FALSE;
		data->text2.color_fg	= data->device->paint.btntext.color;
		data->text2.enable		= TRUE;

		InvalidateRect(control, NULL, TRUE);
	}
	else
	{
		data->arrow.brush		= data->device->paint.btnshadow.brush;
		data->arrow.pen			= data->device->paint.btnshadow.pen;

		data->text1.color_fg	= data->device->paint.btnhighlt.color;
		data->text1.enable		= TRUE;
		data->text2.color_fg	= data->device->paint.btnshadow.color;
		data->text2.enable		= TRUE;

		InvalidateRect(control, NULL, TRUE);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_paint
*
*	Purpose:
*
*		Process WM_PAINT messages.
*
*	Arguments:
*
*		control		The control handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_paint(HWND control)
{
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);
	PAINTSTRUCT	ps;

	if (data)
	{
		data->dc	= BeginPaint(control, &ps);
		_control_paint(control, data);
		EndPaint(control, &ps);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_control_proc
*
*	Purpose:
*
*		Processes messages for the control.
*
*	Arguments:
*
*		control		The control handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _control_proc(	HWND	control,
										UINT	msg,
										WPARAM	wParam,
										LPARAM	lParam)
{
	LRESULT		ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(control);
			break;

		case WM_ENABLE:

			ret	= _wm_enable(control, wParam);
			break;

		case WM_PAINT:

			ret	= _wm_paint(control);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	static_init
*
*	Purpose:
*
*		Initialize the custom static control.
*
*	Arguments:
*
*		dlg			The parent dialog handle.
*
*		device		The device structure to access.
*
*		id			The control id.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void static_init(HWND dlg, device_t* device, int id)
{
	HWND		control;
	static_t*	data;
	TEXTMETRIC	tm;

	for (;;)	// A convenience loop.
	{
		control	= GetDlgItem(dlg, id);

		if (control == NULL)
			break;

		data	= (static_t*) malloc(sizeof(static_t));

		if (data == NULL)
			break;

		memset(data, 0, sizeof(static_t));
		SET_WINDOW_USER_DATA(control, data);
		data->dc	= GetDC(dlg);
		GetClientRect(control, &data->client);

		data->device	= device;
		data->style		= (UINT) GetWindowLong(control, GWL_STYLE);

		GetTextMetrics(data->dc, &tm);
		data->tmAscent	= tm.tmAscent;
		data->tmDescent	= tm.tmDescent;

		// Outter border.
		data->border.outter.pen			= data->device->paint.black.pen;
		data->border.outter.count		= 5;
		data->border.outter.list[0].x	= data->client.left;
		data->border.outter.list[0].y	= data->client.top;
		data->border.outter.list[1].x	= data->client.right - 1;
		data->border.outter.list[1].y	= data->client.top;
		data->border.outter.list[2].x	= data->client.right - 1;
		data->border.outter.list[2].y	= data->client.bottom - 1;
		data->border.outter.list[3].x	= data->client.left;
		data->border.outter.list[3].y	= data->client.bottom - 1;
		data->border.outter.list[4].x	= data->client.left;
		data->border.outter.list[4].y	= data->client.top;

		// Interrior border.
		data->border.ridge.pen			= data->device->paint.btnface.pen;
		data->border.ridge.count		= 5;
		data->border.ridge.list[0].x	= data->client.left + 2;
		data->border.ridge.list[0].y	= data->client.top + 2;
		data->border.ridge.list[1].x	= data->client.right - 3;
		data->border.ridge.list[1].y	= data->client.top + 2;
		data->border.ridge.list[2].x	= data->client.right - 3;
		data->border.ridge.list[2].y	= data->client.bottom - 3;
		data->border.ridge.list[3].x	= data->client.left + 2;
		data->border.ridge.list[3].y	= data->client.bottom - 3;
		data->border.ridge.list[4].x	= data->client.left + 2;
		data->border.ridge.list[4].y	= data->client.top + 2;

		// First upper left edge.
		data->border.ul_out.pen			= data->device->paint.btnhighlt.pen;
		data->border.ul_out.count		= 3;
		data->border.ul_out.list[0].x	= data->client.left + 1;
		data->border.ul_out.list[0].y	= data->client.bottom - 3;
		data->border.ul_out.list[1].x	= data->client.left + 1;
		data->border.ul_out.list[1].y	= data->client.top + 1;
		data->border.ul_out.list[2].x	= data->client.right - 2;
		data->border.ul_out.list[2].y	= data->client.top + 1;

		// Second upper left edge.
		data->border.ul_in.pen			= data->device->paint.btnshadow.pen;
		data->border.ul_in.count		= 3;
		data->border.ul_in.list[0].x	= data->client.left + 3;
		data->border.ul_in.list[0].y	= data->client.bottom - 4;
		data->border.ul_in.list[1].x	= data->client.left + 3;
		data->border.ul_in.list[1].y	= data->client.top + 3;
		data->border.ul_in.list[2].x	= data->client.right - 4;
		data->border.ul_in.list[2].y	= data->client.top + 3;

		// First lower right edge.
		data->border.lr_out.pen			= data->device->paint.btnshadow.pen;
		data->border.lr_out.count		= 3;
		data->border.lr_out.list[0].x	= data->client.left + 1;
		data->border.lr_out.list[0].y	= data->client.bottom - 2;
		data->border.lr_out.list[1].x	= data->client.right - 2;
		data->border.lr_out.list[1].y	= data->client.bottom - 2;
		data->border.lr_out.list[2].x	= data->client.right - 2;
		data->border.lr_out.list[2].y	= data->client.top;

		// Second lower right edge.
		data->border.lr_in.pen			= data->device->paint.btnhighlt.pen;
		data->border.lr_in.count		= 3;
		data->border.lr_in.list[0].x	= data->client.left + 3;
		data->border.lr_in.list[0].y	= data->client.bottom - 4;
		data->border.lr_in.list[1].x	= data->client.right - 4;
		data->border.lr_in.list[1].y	= data->client.bottom - 4;
		data->border.lr_in.list[2].x	= data->client.right - 4;
		data->border.lr_in.list[2].y	= data->client.top + 2;

		// Inner rectangle
		data->inside.brush			= data->device->paint.btnface.brush;
		data->inside.pen			= data->device->paint.btnface.pen;
		data->inside.rect.left		= data->client.left + 4;
		data->inside.rect.top		= data->client.top + 4;
		data->inside.rect.right		= data->client.right - 4;
		data->inside.rect.bottom	= data->client.bottom - 4;

		// The arrow head.
		data->arrow.brush		= 0;
		data->arrow.count		= 5;
		data->arrow.direction	= 0;
		data->arrow.enable		= FALSE;
		data->arrow.height		= (GetSystemMetrics(SM_CYSMICON) * 3) / 5;
		data->arrow.offset		= 0;
		data->arrow.pen			= 0;
		data->arrow.width		= (GetSystemMetrics(SM_CXSMICON) * 3) / 5;

		// The text images.
		data->text1.color_bg	= data->device->paint.btnface.color;
		data->text1.color_fg	= data->device->paint.btntext.color;
		data->text1.enable		= TRUE;
		data->text1.font		= device->font.defalt;
		data->text1.rect		= data->inside.rect;
		data->text1.rect.left++;
		data->text1.rect.top++;
		data->text1.rect.right--;
		data->text1.rect.bottom--;
		GetWindowText(control, data->text1.text, sizeof(data->text1.text));
		data->text1.length		= (int) strlen(data->text1.text);

		data->text2				= data->text1;
		data->text2.color_fg	= data->device->paint.btntext.color;

		_text_align(data);
		ReleaseDC(dlg, data->dc);
		SET_WINDOW_WND_PROC(control, _control_proc);
		_wm_enable(control, TRUE);
		break;
	}
}



/*****************************************************************************
*
*	Function:	static_arrow_direction_set
*
*	Purpose:
*
*		Set the control's arrow direction.
*
*	Arguments:
*
*		dlg			The dialog containing the control to access.
*
*		id			The id of the control to access.
*
*		direction	The arrow's direction: - = left, 0 = none, + = right
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void static_arrow_direction_set(HWND dlg, int id, int direction)
{
	HWND		control	= GetDlgItem(dlg, id);
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);
	int			dy		= data->arrow.height / 2;
	int			x		= data->inside.rect.left + data->arrow.offset;
	int			y		= data->inside.rect.top + (data->tmAscent / 2);

	data->arrow.direction	= direction;

	if (direction > 0)
	{
		// The arrow points right.
		data->arrow.enable		= TRUE;
		data->arrow.list[0].x	= x;
		data->arrow.list[0].y	= y;
		data->arrow.list[1].x	= x;
		data->arrow.list[1].y	= y - dy;
		data->arrow.list[2].x	= x + data->arrow.width;
		data->arrow.list[2].y	= y;
		data->arrow.list[3].x	= x;
		data->arrow.list[3].y	= y + dy;
		data->arrow.list[4].x	= x;
		data->arrow.list[4].y	= y;
	}
	else if (direction < 0)
	{
		// The arrow points left.
		data->arrow.enable		= TRUE;
		data->arrow.list[0].x	= x + data->arrow.width;
		data->arrow.list[0].y	= y;
		data->arrow.list[1].x	= x + data->arrow.width;
		data->arrow.list[1].y	= y - dy;
		data->arrow.list[2].x	= x;
		data->arrow.list[2].y	= y;
		data->arrow.list[3].x	= x + data->arrow.width;
		data->arrow.list[3].y	= y + dy;
		data->arrow.list[4].x	= x + data->arrow.width;
		data->arrow.list[4].y	= y;
	}
	else
	{
		// No arrow is utilized.
		data->arrow.enable	= FALSE;
	}
}



/*****************************************************************************
*
*	Function:	static_arrow_offset_set
*
*	Purpose:
*
*		Set the control's arrow direction.
*
*	Arguments:
*
*		dlg			The dialog containing the control to access.
*
*		id			The id of the control to access.
*
*		offset		The offset from the left most text area.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void static_arrow_offset_set(HWND dlg, int id, int offset)
{
	HWND		control	= GetDlgItem(dlg, id);
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);

	data->arrow.offset	= offset;
	static_arrow_direction_set(dlg, id, data->arrow.direction);
}



/*****************************************************************************
*
*	Function:	static_circle_paint
*
*	Purpose:
*
*		Paint a circle given the button's current state.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*		x			The X coordinate of the left edge.
*
*		y			The Y coordinate of the top edge.
*
*		brush		The brush to fill the circle with. Is is the defult used
*					if the device is opened.
*
*		cross		Draw a cross over the circle.
*
*	Returned:
*
*		The width of the output.
*
*****************************************************************************/

int static_circle_paint(
	HWND		control,
	static_t*	data,
	int			x,
	int			y,
	HBRUSH		brush,
	int			cross)
{
	HDC		dc	= GetDC(control);
	HPEN	pen	= data->arrow.pen;
	int		h	= ((data->arrow.height / 2) * 2) + 1;
	int		w	= data->arrow.width + 2;

	if (data->device->handle == NULL)
		brush	= data->arrow.brush;

	brush	= SelectObject(dc, brush);
	pen		= SelectObject(dc, pen);
	y		= (y + (data->tmAscent / 2)) - (h / 2);

	Ellipse(dc, x, y, x + w, y + h);

	if (cross)
	{
		MoveToEx(dc, x, y, NULL);
		LineTo(dc, x + w, y + h);
		MoveToEx(dc, x, y + h - 1, NULL);
		LineTo(dc, x + w, y - 1);
	}

	SelectObject(dc, brush);
	SelectObject(dc, pen);
	ReleaseDC(control, dc);
	return(w);
}



/*****************************************************************************
*
*	Function:	static_paint_function_set
*
*	Purpose:
*
*		Call the given function when the control needs painting. The function
*		is called after the common portion is painted.
*
*	Arguments:
*
*		dlg			The dialog containing the control to access.
*
*		id			The id of the control to access.
*
*		paint		The function to call.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void static_paint_function_set(
	HWND	dlg,
	int		id,
	void	(*paint)(HWND control, static_t* data))
{
	HWND		control	= GetDlgItem(dlg, id);
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);

	data->paint	= paint;
}



/*****************************************************************************
*
*	Function:	static_text_paint
*
*	Purpose:
*
*		Paint text per the given structure.
*
*	Arguments:
*
*		dlg		The control to access.
*
*		data	The control specific datw e track.
*
*		x		The X coordinate.
*
*		y		The Y coordinate.
*
*		text	The text to paint.
*
*	Returned:
*
*		The width of the text painted.
*
*****************************************************************************/

int static_text_paint(HWND control, static_t* data, int x, int y, const char* text)
{
	HDC				dc		= GetDC(control);
	static_text_t	t;
	int				w;

	t	= data->text1;
	strncpy(t.text, text, sizeof(t.text));
	t.length	= (int) strlen(t.text);
	t.x			= x + 1;
	t.y			= y + 1;
	_text_paint(dc, &t);

	t	= data->text2;
	strncpy(t.text, text, sizeof(t.text));
	t.length	= (int) strlen(t.text);
	t.x			= x;
	t.y			= y;
	w			= _text_paint(dc, &t);

	ReleaseDC(control, dc);
	return(w);
}



/*****************************************************************************
*
*	Function:	static_text_width_get
*
*	Purpose:
*
*		Return the width, in pixels, of the control's text.
*
*	Arguments:
*
*		dlg		The dialog containing the control to access.
*
*		id		The id of the control to access.
*
*	Returned:
*
*		The computed width in pixels.
*
*****************************************************************************/

int static_text_width_get(HWND dlg, int id)
{
	HWND		control	= GetDlgItem(dlg, id);
	static_t*	data	= (static_t*) GET_WINDOW_USER_DATA(control);
	HDC			dc;
	HFONT		font_old;
	RECT		rect;
	int			width;

	dc			= GetDC(control);
	font_old	= SelectObject(dc, data->text1.font);
	memset(&rect, 0, sizeof(rect));
	DrawText(dc, data->text1.text, -1, &rect, DT_CALCRECT);
	width	= RECT_WIDTH(rect);
	SelectObject(dc, font_old);
	ReleaseDC(control, dc);
	return(width);
}



/*****************************************************************************
*
*	Function:	static_wave_paint
*
*	Purpose:
*
*		Paint the given waveform using the control's current state.
*
*	Arguments:
*
*		data		The control specific data we track.
*
*		x			The starting X coordinate.
*
*		y			The starting Y coordinate. This is the top edge, but we
*					modify this so the centerline aligns with that of the
*					arrow, if one is present.
*
*		wave		The data set to process.
*
*	Returned:
*
*		The width of the waveform.
*
*****************************************************************************/

int static_wave_paint(
	static_t*		data,
	int				x,
	int				y,
	const wave_t*	list)
{
	int		dx;
	int		dy;
	HPEN	pen_old;
	int		width;
	int		x1;
	int		x2;

	dx		= data->arrow.width / 3;
	dy		= (data->arrow.height / 2) * 2;
	y		= (y + (data->tmAscent / 2)) - (dy / 2);
	pen_old	= SelectObject(data->dc, data->arrow.pen);
	x1		= x;
	x2		= x;

	for (; list->count > 0; list++)
	{
		if (list->flags & WAVE_TOP)
			x2	= _wave_paint(data, x, y, list, dx, dy);

		if (list->flags & WAVE_BOTTOM)
			x2	= _wave_paint(data, x, y + dy, list, dx, -dy);

		x	= x2;
	}

	SelectObject(data->dc, pen_old);
	width	= x - x1;
	return(width);
}


